@@ -38,11 +38,13 @@ struct feed {
 	char * link;
 	char * description;
 	char * lastmodified;					/* Content of header as sent by the server. */
+	int lasthttpstatus;
 	struct newsitem * items;
 	struct feed *next_ptr, *prev_ptr;
 	int problem;							/* Set if there was a problem downloading the feed. */
-	char *override;							/* Custom feed title. */
-	char *original;							/* Original feed title. */
+	char * override;						/* Custom feed title. */
+	char * original;						/* Original feed title. */
+	struct feedcategories * feedcategories;
 };
 
 struct newsitem {
@@ -68,14 +70,29 @@ struct keybindings {
 	char reload;
 	char reloadall;
 	char urljump;
+	char urljump2;
 	char changefeedname;
 	char sortfeeds;
 	char pdown;
 	char pup;
+	char categorize;
+	char filter;
+	char filtercurrent;
+	char nofilter;
+	char help;
+	char about;
+};
+
+/* Color definitions */
+struct color {
+	int changed;							/* Set to 1 when changing anything color related (labels) to avoid unneccessary writing of the config file. */
+	int newitems;
+	int newitemsbold;
+	int urljump;
+	int urljumpbold;
 };
 
 extern struct feed *first_ptr;
-extern int lasthttpstatus;					/* Global defs. Should be coded more cleanly. */
 
 #ifdef LOCALEPATH
 #	include <libintl.h>
