@@ -32,6 +32,11 @@
 #include "conversions.h"
 #include "config.h"
 #include "interface.h"
+#include "ui-support.h"
+
+#ifdef SUN
+#	include "os-support.h"
+#endif
 
 char * iconvert (char * inbuf, char * from, char * to) {
 	iconv_t cd;							/* Iconvs conversion descriptor. */
