@@ -32,9 +32,13 @@
 #include "config.h"
 #include "interface.h"
 #include "main.h"
+#include "categories.h"
+#include "ui-support.h"
 
 extern char *browser;
 extern struct keybindings keybindings;
+extern struct categories *first_category;
+extern struct color color;
  
 /* Snowflake for the xmas about box. */
 struct snowflake {
@@ -74,8 +78,8 @@ char * UIOneLineEntryField (int x, int y) {
 }
 
 int UIChangeBrowser (void) {
-	int centerx, centery, i, j, len;
 	char *browserstring;
+	int len;
 	
 	/* malloc = 17 (strlen("Current setting: ") + browser
 	   We will malloc a bigger junk, because other languages
@@ -83,18 +87,11 @@ int UIChangeBrowser (void) {
 	len = strlen(_("Current setting: %s")) + strlen(browser) + 1;
 	browserstring = malloc (len);
 	snprintf (browserstring, len, _("Current setting: %s"), browser);
-	
-	centerx = COLS / 2;
-	centery = LINES / 2;
 
 	/* Clear screen area we want to "draw" to. */
 	attron (WA_REVERSE);
-	for (i = 5; i <= 7; i++) {
-		for (j = -(COLS/2-3); j <= (COLS/2-3); j++) {
-				mvaddch (i, centerx+j, ' ');
-		}	
-	}
-	
+	UISupportDrawBox (3, 5, COLS-4, 7);
+		
 	UIStatus (browserstring, 0);
 	free (browserstring);
 	
@@ -120,18 +117,11 @@ int UIChangeBrowser (void) {
 */
 int UIChangeFeedName (struct feed *cur_ptr) {
 	char *newname;
-	int centerx, centery, i, j;
-
-	centerx = COLS / 2;
-	centery = LINES / 2;
 
 	/* Clear screen area we want to "draw" to. */
 	attron (WA_REVERSE);
-	for (i = 5; i <= 7; i++) {
-		for (j = -(COLS/2-3); j <= (COLS/2-3); j++) {
-				mvaddch (i, centerx+j, ' ');
-		}	
-	}
+	UISupportDrawBox (3, 5, COLS-4, 7);
+
 	UIStatus (_("Enter new name. Blank line to abort. '-' to reset."), 0);
 	
 	newname = UIOneLineEntryField (5, 6);
@@ -180,21 +170,13 @@ int UIChangeFeedName (struct feed *cur_ptr) {
 int UIAddFeed (void) {
 	char tmp[512];
 	char *url;
-	int centerx, centery;
-	int i, j;
 	struct feed *new_ptr;
 	struct feed *tmp_ptr;
 
-	centerx = COLS / 2;
-	centery = LINES / 2;
-
 	/* Clear screen area we want to "draw" to. */
 	attron (WA_REVERSE);
-	for (i = 5; i <= 7; i++) {
-		for (j = -(COLS/2-3); j <= (COLS/2-3); j++) {
-				mvaddch (i, centerx+j, ' ');
-		}	
-	}
+	UISupportDrawBox (3, 5, COLS-4, 7);
+	
 	UIStatus (_("Enter URL of the feed you want to add. Blank line to abort."), 0);
 
 	url = UIOneLineEntryField (5, 6);
@@ -226,7 +208,8 @@ int UIAddFeed (void) {
 	new_ptr->override = NULL;
 	new_ptr->original = NULL;
 	new_ptr->problem = 0;
-			
+	new_ptr->feedcategories = NULL;
+	
 	/* Attach to feed pointer chain. */
 	strncpy (new_ptr->feedurl, url, strlen(url)+1);
 	new_ptr->next_ptr = first_ptr;
@@ -240,8 +223,8 @@ int UIAddFeed (void) {
 
 	/* Download new feed and DeXMLize it. */	
 	if ((UpdateFeed (new_ptr)) != 0) {
-		if ((lasthttpstatus != 200) && (lasthttpstatus != 0)) {
-			snprintf (tmp, sizeof(tmp), _("HTTP error %d - Could not update feed."), lasthttpstatus);
+		if ((new_ptr->lasthttpstatus != 200) && (new_ptr->lasthttpstatus != 0)) {
+			snprintf (tmp, sizeof(tmp), _("HTTP error %d - Could not update feed."), new_ptr->lasthttpstatus);
 			UIStatus (tmp, 2);
 		}
 		
@@ -269,10 +252,12 @@ int UIAddFeed (void) {
 }
 
 void FeedInfo (struct feed * current_feed) {
-	int centerx, i, j, len;
+	int centerx, len;
 	char *hashme;			/* Hashed filename. */
 	char *file;
+	char *categories = NULL;
 	struct stat filetest;
+	struct feedcategories *category;
 	
 	centerx = COLS / 2;
 	
@@ -284,20 +269,9 @@ void FeedInfo (struct feed * current_feed) {
 	file = malloc (len);
 	snprintf (file, len, "%s/.snownews/cache/%s", getenv("HOME"), hashme);
 
-	/* Clear the help screen area and print the text on reverse background. */
-	/*attron (WA_REVERSE);
-	for (i = -4; i <= 5; i++) {
-		for (j = -20; j <= 20; j++) {
-				mvaddch (centery+i, centerx+j, ' ');
-		}	
-	}*/
-	/* Clear screen area we want to "draw" to. */
+	UISupportDrawBox (5, 4, COLS-6, 12);
+	
 	attron (WA_REVERSE);
-	for (i = 0; i <= 7; i++) {
-		for (j = -(COLS/2-5); j <= (COLS/2-5); j++) {
-				mvaddch (4+i, centerx+j, ' ');
-		}	
-	}
 	mvaddstr (5, centerx-((strlen(current_feed->title))/2), current_feed->title);
 	mvaddstr (7, centerx-(COLS/2-7), current_feed->feedurl);
 	if (current_feed->lastmodified != NULL)
@@ -309,6 +283,52 @@ void FeedInfo (struct feed * current_feed) {
 		mvprintw (9, centerx-(COLS/2-7), _("In disk cache: %lld bytes"), (long long int) filetest.st_size);
 	} else
 		mvaddstr (9, centerx-(COLS/2-7), _("Not in disk cache."));
+		
+	/* Print category info */
+	mvaddstr (10, centerx-(COLS/2-7), _("Categories:"));
+	if (current_feed->feedcategories == NULL)
+		mvaddstr (10, centerx-(COLS/2-7)+strlen(_("Categories:"))+1, "none");
+	else {
+		categories = malloc(1);
+		memset (categories, 0, 1);
+		len = 0;
+		for (category = current_feed->feedcategories; category != NULL; category = category->next_ptr) {
+			len += strlen(category->name)+4;
+			categories = realloc (categories, len);
+			strncat (categories, category->name, len);
+			if (category->next_ptr != NULL)
+				strcat (categories, ", ");
+		}
+		mvprintw (10, centerx-(COLS/2-7)+strlen(_("Categories:"))+1, categories);
+		if (categories != NULL)
+			free (categories);
+	}
+	
+	/* Add a smiley indicator to the http status telling the overall status
+	   so you don't have to know what the HTTP return codes mean.
+	   Yes I think I got the idea from cdparanoia. :) */
+	if (current_feed->lasthttpstatus != 0) {
+		mvprintw (11, centerx-(COLS/2-7), _("Last webserver status: %d"), current_feed->lasthttpstatus);
+		
+		switch (current_feed->lasthttpstatus) {
+			case 200:
+			case 304:
+				mvaddstr (11, centerx-(COLS/2-7)+strlen(_("Last webserver status: %d"))+2, ":-)");
+				break;
+			case 403:
+			case 500:
+			case 503:
+				mvaddstr (11, centerx-(COLS/2-7)+strlen(_("Last webserver status: %d"))+2, ":-P");
+				break;
+			case 404:
+			case 410:
+				mvaddstr (11, centerx-(COLS/2-7)+strlen(_("Last webserver status: %d"))+2, ":-(");
+				break;
+			default:
+				mvaddstr (11, centerx-(COLS/2-7)+strlen(_("Last webserver status: %d"))+2, "8-/");
+				break;
+		}
+	}
 	
 	UIStatus (_("Displaying feed information."), 0);
 	
@@ -319,15 +339,10 @@ void FeedInfo (struct feed * current_feed) {
 }
 
 int UIDeleteFeed (char * feedname) {
-	int i, j;
-
-	/* Clear screen area we want to "draw" to. */
+	
+	UISupportDrawBox (3, 5, COLS-3, 8);
+	
 	attron (WA_REVERSE);
-	for (i = 5; i <= 8; i++) {
-		for (j = -(COLS/2-3); j <= (COLS/2-3); j++) {
-				mvaddch (i, (COLS/2)+j, ' ');
-		}	
-	}
 	mvaddstr (6, COLS/2-21, _("Are you sure you want to delete this feed?"));
 	mvprintw (7, 5, "%s", feedname);
 	
@@ -341,31 +356,30 @@ int UIDeleteFeed (char * feedname) {
 
 void UIHelpScreen (void) {
 	int centerx, centery;				/* Screen center x/y coordinate. */
-	int i, j;
 	int userinput;
 	
 	centerx = COLS / 2;
 	centery = LINES / 2;
-
-	/* Clear the help screen area and print the text on reverse background. */
+	
+	UISupportDrawBox ((COLS/2)-20, (LINES/2)-9, (COLS/2)+24, (LINES/2)+8);
+	
 	attron (WA_REVERSE);
-	for (i = -6; i <= 7; i++) {
-		for (j = -20; j <= 20; j++) {
-				mvaddch (centery+i, centerx+j, ' ');
-		}	
-	}
-	mvprintw (centery-5, centerx-18, _("%c:    Add RSS feed..."), keybindings.addfeed);	
-	mvprintw (centery-4, centerx-18, _("%c:    Delete highlighted RSS feed..."), keybindings.deletefeed);
-	mvprintw (centery-3, centerx-18, _("%c:    Rename feed..."), keybindings.changefeedname);
-	mvprintw (centery-2, centerx-18, _("%c:    Reload all feeds"), keybindings.reloadall);
-	mvprintw (centery-1, centerx-18, _("%c:    Reload this feed"), keybindings.reload);
-	mvprintw (centery,   centerx-18, _("%c:    Mark all read"), keybindings.markread);
-	mvprintw (centery+1, centerx-18, _("%c:    Change default browser..."), keybindings.dfltbrowser);
-	mvprintw (centery+2, centerx-18, _("%c, %c: Move item up, down"), keybindings.moveup, keybindings.movedown);
-	mvprintw (centery+3, centerx-18, _("%c:    Sort feed list alphabetically"), keybindings.sortfeeds);
-	mvaddstr (centery+4, centerx-18, _("tab:  Type Ahead Find"));
-	mvaddstr (centery+5, centerx-18, _("A:    About"));
-	mvprintw (centery+6, centerx-18, _("%c:    Quit program"), keybindings.quit);
+	mvprintw (centery-8, centerx-18, _("%c:    Add RSS feed..."), keybindings.addfeed);	
+	mvprintw (centery-7, centerx-18, _("%c:    Delete highlighted RSS feed..."), keybindings.deletefeed);
+	mvprintw (centery-6, centerx-18, _("%c:    Rename feed..."), keybindings.changefeedname);
+	mvprintw (centery-5, centerx-18, _("%c:    Reload all feeds"), keybindings.reloadall);
+	mvprintw (centery-4, centerx-18, _("%c:    Reload this feed"), keybindings.reload);
+	mvprintw (centery-3, centerx-18, _("%c:    Mark all read"), keybindings.markread);
+	mvprintw (centery-2, centerx-18, _("%c:    Change default browser..."), keybindings.dfltbrowser);
+	mvprintw (centery-1, centerx-18, _("%c, %c: Move item up, down"), keybindings.moveup, keybindings.movedown);
+	mvprintw (centery,   centerx-18, _("%c:    Sort feed list alphabetically"), keybindings.sortfeeds);
+	mvprintw (centery+1, centerx-18, _("%c:    Categorize feed..."), keybindings.categorize);
+	mvprintw (centery+2, centerx-18, _("%c:    Apply filter..."), keybindings.filter);
+	mvprintw (centery+3, centerx-18, _("%c:    Only current category"), keybindings.filtercurrent);
+	mvprintw (centery+4, centerx-18, _("%c:    Remove filter"), keybindings.nofilter);
+	mvaddstr (centery+5, centerx-18, _("tab:  Type Ahead Find"));
+	mvaddstr (centery+6, centerx-18, _("A:    About"));
+	mvprintw (centery+7, centerx-18, _("%c:    Quit program"), keybindings.quit);
 	attroff (WA_REVERSE);
 
 	UIStatus (_("Press the any(tm) key to exit help screen."), 0);
@@ -379,29 +393,24 @@ void UIHelpScreen (void) {
 
 void UIDisplayFeedHelp (void) {
 	int centerx, centery;				/* Screen center x/y coordinate. */
-	int i, j;
 	int userinput;
 	
 	centerx = COLS / 2;
 	centery = LINES / 2;
-
-	/* Clear the help screen area and print the text on reverse background. */
+	
+	UISupportDrawBox ((COLS/2)-20, (LINES/2)-5, (COLS/2)+20, (LINES/2)+6);
+	
 	attron (WA_REVERSE);
-	for (i = -5; i <= 6; i++) {
-		for (j = -19; j <= 19; j++) {
-				mvaddch (centery+i, centerx+j, ' ');
-		}	
-	}
-	mvprintw (centery-4, centerx-17, _("%c, up:    Previous item"), keybindings.prev);	
-	mvprintw (centery-3, centerx-17, _("%c, down:  Next item"), keybindings.next);
-	mvaddstr (centery-2, centerx-17, _("enter:    View item"));
-	mvprintw (centery-1, centerx-17, _("%c:        Reload this feed"), keybindings.reload);
-	mvprintw (centery,   centerx-17, _("%c:        Open homepage"), keybindings.urljump);
-	mvprintw (centery+1, centerx-17, _("%c:        Mark all read"), keybindings.markread);
-	mvprintw (centery+2, centerx-17, _("%c:        Show feed info..."), keybindings.feedinfo);
-	mvaddstr (centery+3, centerx-17, _("tab:      Type Ahead Find"));
-	mvaddstr (centery+4, centerx-17, _("A:        About"));
-	mvprintw (centery+5, centerx-17, _("%c:        Return to main menu"), keybindings.quit);
+	mvprintw (centery-4, centerx-18, _("%c, up:    Previous item"), keybindings.prev);	
+	mvprintw (centery-3, centerx-18, _("%c, down:  Next item"), keybindings.next);
+	mvaddstr (centery-2, centerx-18, _("enter:    View item"));
+	mvprintw (centery-1, centerx-18, _("%c:        Reload this feed"), keybindings.reload);
+	mvprintw (centery,   centerx-18, _("%c:        Open homepage"), keybindings.urljump);
+	mvprintw (centery+1, centerx-18, _("%c:        Open link"), keybindings.urljump2);
+	mvprintw (centery+2, centerx-18, _("%c:        Mark all read"), keybindings.markread);
+	mvprintw (centery+3, centerx-18, _("%c:        Show feed info..."), keybindings.feedinfo);
+	mvaddstr (centery+4, centerx-18, _("tab:      Type Ahead Find"));
+	mvprintw (centery+5, centerx-18, _("%c:        Return to main menu"), keybindings.quit);
 	attroff (WA_REVERSE);
 
 	UIStatus (_("Press the any(tm) key to exit help screen."), 0);
@@ -412,25 +421,19 @@ void UIDisplayFeedHelp (void) {
 
 void UIDisplayItemHelp (void) {
 	int centerx, centery;				/* Screen center x/y coordinate. */
-	int i, j;
 	int userinput;
 	
 	centerx = COLS / 2;
 	centery = LINES / 2;
-
-	/* Clear the help screen area and print the text on reverse background. */
+	
+	UISupportDrawBox ((COLS/2)-18, (LINES/2)-3, (COLS/2)+18, (LINES/2)+3);
+	
 	attron (WA_REVERSE);
-	for (i = -3; i <= 4; i++) {
-		for (j = -18; j <= 18; j++) {
-				mvaddch (centery+i, centerx+j, ' ');
-		}	
-	}
 	mvaddstr (centery-2, centerx-16, _("up, down: Scroll text"));
 	mvprintw (centery-1, centerx-16, _("%c, left:  Previous item"), keybindings.prev);	
 	mvprintw (centery,   centerx-16, _("%c, right: Next item"), keybindings.next);
-	mvprintw (centery+1, centerx-16, _("%c:        Open link..."), keybindings.urljump);
-	mvaddstr (centery+2, centerx-16, _("A:        About"));
-	mvprintw (centery+3, centerx-16, _("%c, enter: Return to overview"), keybindings.quit);
+	mvprintw (centery+1, centerx-16, _("%c:        Open link"), keybindings.urljump);
+	mvprintw (centery+2, centerx-16, _("%c, enter: Return to overview"), keybindings.quit);
 	attroff (WA_REVERSE);
 
 	UIStatus (_("Press the any(tm) key to exit help screen."), 0);
@@ -439,6 +442,30 @@ void UIDisplayItemHelp (void) {
 		ungetch(userinput);
 }
 
+/* This draws the colored parts of the christmas tree.
+   Called everytime during SnowFall to avoid to much duplicate code.
+   Use sparsly as snowflakes will vanish behind this elements. */
+void ChristmasTree (void) {
+	attron (COLOR_PAIR(13));
+	mvaddch (6, 13, '&');
+	mvaddch (16, 5, '&');
+	attroff (COLOR_PAIR(13));
+	attron (COLOR_PAIR(10));
+	mvaddch (9, 10, '&');
+	mvaddch (19, 12, '&');
+	attroff (COLOR_PAIR(10));
+	attron (COLOR_PAIR(15));
+	mvaddch (12, 14, '&');
+	mvaddch (18, 6, '&');
+	attroff (COLOR_PAIR(15));
+	attron (COLOR_PAIR(14));
+	mvaddch (16, 15, '&');
+	mvaddch (13, 10, '&');
+	attroff (COLOR_PAIR(14));
+}
+
+/* This function sometimes deletes characters. Maybe oldchar doesn't
+   get saved correctly everytime. */
 void Snowfall (void) {
 	struct snowflake *cur;
 	struct snowflake *first = NULL;
@@ -460,6 +487,11 @@ void Snowfall (void) {
 	/* Set ncurses halfdelay mode. */
 		halfdelay (3);
 	
+	/* White snowflakes! */
+	/* Doesn't work on white terminal background... obviously.
+	attron (COLOR_PAIR(16));
+	attron (WA_BOLD); */
+	
 	while (1) {
 		/* Set up the storm. */
 		if (windtimer == 0) {
@@ -517,8 +549,25 @@ void Snowfall (void) {
 			/* Draw every snowflake at its coordinates to the screen. */
 			if (cur->visible) {
 				/* Only draw if y<=LINES. This makes some snow lie on bottom of screen. */
-				if (cur->y <= LINES)
+				if (cur->y <= LINES) {
+					/* Restore old char with proper color.
+					   See also ChristmasTree() above. */
+					if (((cur->oldx < 25) && (cur->oldy > 5)) ||
+						(cur->oldx < 20))
+						attron (COLOR_PAIR(11));
+					if ((cur->oldx < 14) && (cur->oldx > 9) && (cur->oldy < 24) && (cur->oldy > 20)) {
+						attroff (COLOR_PAIR(11));
+						attron (COLOR_PAIR(12));
+					}
 					mvaddch (cur->oldy, cur->oldx, cur->oldchar2);
+					if (((cur->oldx < 25) && (cur->oldy > 5)) ||
+						(cur->oldx < 20))
+						attroff (COLOR_PAIR(11));
+					if ((cur->oldx < 14) && (cur->oldx > 9) && (cur->oldy < 24) && (cur->oldy > 20)) {
+						attroff (COLOR_PAIR(12));
+					}
+					ChristmasTree();
+				}
 				mvaddch (cur->y, cur->x, '*');
 				cur->oldx = cur->x;
 				cur->oldy = cur->y;
@@ -583,12 +632,11 @@ void Snowfall (void) {
 void UIAbout (void) {
 	struct tm *t;
 	time_t tunix;
-	int ypos, xpos;
+	int xpos;
 	
 	clear();				/* Get the crap off the screen to make room
 							   for our wonderful ASCII logo. :) */
 
-	ypos = 2;
 	xpos = COLS/2 - 40;
 	
 	if (COLS < 80) {
@@ -606,11 +654,17 @@ void UIAbout (void) {
 	
 	/* Careful with tm_mon==(0-11) vs. tm_mday==(1-31)  STUPID! */
 	if ((t->tm_mon == 11) && (t->tm_mday >= 24) && (t->tm_mday <= 26)) {
-		/* Christmas tree. */
+		/* Logo */
 		mvaddstr (1,  0, "                      _____ _____    ____ _______ _____   ____ _______   _____");
 		mvaddstr (2,  0, "                     /  __/ \\    \\  / __ \\\\ |  | \\\\    \\ /  _/ \\ |  | \\ /  __/");
-		mvaddstr (3,  0, "          _\\/_       \\___ \\  \\ |  \\ \\ \\/ / \\     / \\ |  \\\\ __\\  \\     / \\___ \\");
-		mvaddstr (4,  0, "           /\\        /____/  /_|__/  \\__/  /__|_/  /_|__/ \\___\\ /__|_/  /____/");
+		mvaddstr (3,  0, "                     \\___ \\  \\ |  \\ \\ \\/ / \\     / \\ |  \\\\ __\\  \\     / \\___ \\");
+		mvaddstr (4,  0, "                     /____/  /_|__/  \\__/  /__|_/  /_|__/ \\___\\ /__|_/  /____/");
+		/* Christmas tree. */
+		attron (COLOR_PAIR(11));
+		mvaddstr (1,  0, "");
+		mvaddstr (2,  0, "");
+		mvaddstr (3,  0, "          _\\/_");
+		mvaddstr (4,  0, "           /\\");
 		mvaddstr (5,  0, "          /  \\");
 		mvaddstr (6,  0, "         /   &\\");
 		mvaddstr (7,  0, "        /      \\");
@@ -630,41 +684,237 @@ void UIAbout (void) {
 		mvaddstr (21, 0, "   /______####_\\_____\\");
 		mvaddstr (22, 0, "         |####|");
 		mvaddstr (23, 0, "         |####|");
+		attroff (COLOR_PAIR(11));
+		attron (COLOR_PAIR(12));
+		mvaddstr (21, 10, "####");
+		mvaddstr (22, 10, "####");
+		mvaddstr (23, 10, "####");
+		attroff (COLOR_PAIR(12));
+		ChristmasTree();
 		/* Credits. */
 		mvprintw (5, 21, "Version %s", VERSION);
-		mvprintw (8, 35, _("Merry Christmas from the Snownews developers."));
-		mvaddstr (10, 35, _("Main code"));
-		mvaddstr (11, 35, "Oliver Feiler");
-		mvaddstr (13, 35, _("Additional Code"));
-		mvaddstr (14, 35, "Rene Puls");
-		mvaddstr (16, 35, _("Translation team"));
-		mvaddstr (17, 35, "Oliver Feiler, Frank van der Loo,");
-		mvaddstr (18, 35, "Pascal Varet, Simon Isakovic");
-		mvaddstr (19, 35, "Fernando J. Pereda");
+		mvprintw (8, 29, _("Merry Christmas from the Snownews developers."));
+		mvaddstr (10, 29, _("Main code"));
+		mvaddstr (11, 29, "Oliver Feiler");
+		mvaddstr (13, 29, _("Additional Code"));
+		mvaddstr (14, 29, "Rene Puls");
+		mvaddstr (16, 29, _("Translation team"));
+		mvaddstr (17, 29, "Oliver Feiler, Frank van der Loo,");
+		mvaddstr (18, 29, "Pascal Varet, Simon Isakovic");
+		mvaddstr (19, 29, "Fernando J. Pereda, Marco Cova");
 		
 		Snowfall();
 	} else {
 		/* 80 COLS logo */
-		mvaddstr (ypos,   xpos, "  ________ _______     ____ __________ _______     ______ __________   ________");
-		mvaddstr (ypos+1, xpos, " /  _____/ \\      \\   /    \\\\  |    | \\\\      \\   /  ___/ \\  |    | \\ / ______/");
-		mvaddstr (ypos+2, xpos, " \\____  \\   \\   |  \\ /  /\\  \\\\   |    / \\   |  \\ /     /   \\   |    / \\____  \\");
-		mvaddstr (ypos+3, xpos, " /       \\  /   |   \\\\  \\/  / \\  |   /  /   |   \\\\  ___\\    \\  |   /  /       \\");
-		mvaddstr (ypos+4, xpos, "/______  / / ___|___/ \\____/  /__|__/  / ___|___/ \\____ \\   /__|__/  /______  /");
-		mvaddstr (ypos+5, xpos, "       \\/  \\/                          \\/              \\/                   \\/");
-		mvprintw (ypos+7, COLS/2-(strlen("Version")+strlen(VERSION)+1)/2, "Version %s", VERSION);
+		mvaddstr (2,   xpos, "  ________ _______     ____ __________ _______     ______ __________   ________");
+		mvaddstr (3, xpos, " /  _____/ \\      \\   /    \\\\  |    | \\\\      \\   /  ___/ \\  |    | \\ / ______/");
+		mvaddstr (4, xpos, " \\____  \\   \\   |  \\ /  /\\  \\\\   |    / \\   |  \\ /     /   \\   |    / \\____  \\");
+		mvaddstr (5, xpos, " /       \\  /   |   \\\\  \\/  / \\  |   /  /   |   \\\\  ___\\    \\  |   /  /       \\");
+		mvaddstr (6, xpos, "/______  / / ___|___/ \\____/  /__|__/  / ___|___/ \\____ \\   /__|__/  /______  /");
+		mvaddstr (7, xpos, "       \\/  \\/                          \\/              \\/                   \\/");
+		mvprintw (9, COLS/2-(strlen("Version")+strlen(VERSION)+1)/2, "Version %s", VERSION);
 	
-		mvaddstr (ypos+10, COLS/2-(strlen(_("Brought to you by:")))/2, _("Brought to you by:"));
-		mvaddstr (ypos+12, COLS/2-(strlen(_("Main code")))/2, _("Main code"));
-		mvaddstr (ypos+13, COLS/2-6, "Oliver Feiler");
-		mvaddstr (ypos+15, COLS/2-(strlen(_("Additional code")))/2, _("Additional code"));
-		mvaddstr (ypos+16, COLS/2-4, "Rene Puls");
-		mvaddstr (ypos+18, COLS/2-(strlen(_("Translation team")))/2, _("Translation team"));
-		mvaddstr (ypos+19, COLS/2-31, "Oliver Feiler, Frank van der Loo, Pascal Varet, Simon Isakovic,");
-		mvaddstr (ypos+20, COLS/2-9, "Fernando J. Pereda");
+		mvaddstr (12, COLS/2-(strlen(_("Brought to you by:")))/2, _("Brought to you by:"));
+		mvaddstr (14, COLS/2-(strlen(_("Main code")))/2, _("Main code"));
+		mvaddstr (15, COLS/2-6, "Oliver Feiler");
+		mvaddstr (17, COLS/2-(strlen(_("Additional code")))/2, _("Additional code"));
+		mvaddstr (18, COLS/2-4, "Rene Puls");
+		mvaddstr (20, COLS/2-(strlen(_("Translation team")))/2, _("Translation team"));
+		mvaddstr (21, COLS/2-31, "Oliver Feiler, Frank van der Loo, Pascal Varet, Simon Isakovic,");
+		mvaddstr (22, COLS/2-15, "Fernando J. Pereda, Marco Cova");
 
 		refresh();
 		getch();
 	}
 }
 
- 
+/* Add/remove categories for given feed. This takes over the main interface while running. */
+void CategorizeFeed (struct feed * current_feed) {
+	int i, n, nglobal, y;
+	int uiinput;
+	char *newcategory;
+	struct feedcategories *category;
+	struct categories *cur_ptr;
+	
+	/* Return if we got passed a NULL pointer (no feeds added to main program). */
+	if (current_feed == NULL)
+		return;
+		
+	/* Determine number of global categories. */
+	nglobal = 0;
+	for (cur_ptr = first_category; cur_ptr != NULL; cur_ptr = cur_ptr->next_ptr) {
+		nglobal++;
+	}
+
+	/* We're taking over the program! */
+	while (1) {
+		/* Determine number of categories for current_feed. */
+		n = 0;
+		for (category = current_feed->feedcategories; category != NULL; category = category->next_ptr) {
+			n++;
+		}
+
+		UISupportDrawBox ((COLS/2)-37, 2, (COLS/2)+37, LINES-3);
+		
+		attron (WA_REVERSE);
+		mvprintw (3, (COLS/2)-((strlen(_("Category configuration for %s"))+strlen(current_feed->title))/2),
+			_("Category configuration for %s"), current_feed->title);
+		
+
+		/* No category defined yet */
+		if (current_feed->feedcategories == NULL) {
+			i = 49;
+			y = 5;
+			mvaddstr (y, (COLS/2)-33, _("No category defined. Select one or add a new category:"));
+			for (cur_ptr = first_category; cur_ptr != NULL; cur_ptr = cur_ptr->next_ptr) {
+				mvprintw (y+1, (COLS/2)-33, "%c. %s", i, cur_ptr->name);
+				y++;
+				i++;
+				/* Fast forward to 'a' if we hit ASCII 58 ':' */
+				if (i == 58)
+					i += 39;
+			}
+		
+			UIStatus (_("Select category number to add, press 'A' to add a new category or 'q' to quit."), 0);
+		} else {
+			i = 49;
+			y = 5;
+			mvaddstr (y, (COLS/2)-33, _("Categories defined for this feed:"));
+			for (category = current_feed->feedcategories; category != NULL; category = category->next_ptr) {
+				mvprintw (y+1, (COLS/2)-33, "%c. %s", i, category->name);
+				y++;
+				i++;
+				/* Fast forward to 'a' if we hit ASCII 58 ':' */
+				if (i == 58)
+					i += 39;
+			}
+	
+			UIStatus (_("Select a category number to delete, press 'A' to add a new one or 'q' to quit."), 0);
+		}
+		
+		refresh();
+		
+		uiinput = getch();
+		if (uiinput == keybindings.quit)
+			return;
+		if (uiinput == 'A') {
+			/* Clear screen area we want to "draw" to. */
+			UISupportDrawBox ((COLS/2)-37, 5, (COLS/2)+37, LINES-3);
+			
+			attron (WA_REVERSE);
+			UIStatus (_("Enter new category name."), 0);
+			newcategory = UIOneLineEntryField ((COLS/2)-33, 5);
+			if (strlen(newcategory) == 0)
+				UIStatus (_("Aborted."), 1);
+			else {
+				if (!FeedCategoryExists (current_feed, newcategory))
+					FeedCategoryAdd (current_feed, newcategory);
+				else
+					UIStatus (_("Category already defined for this feed!"), 1);
+			}
+			free (newcategory);
+		}
+		/* If uiinput is 1-9,a-z
+		   If uiinput < i (ASCII of max selectable element) ignore event. */
+		if ((((uiinput >= 49) && (uiinput <= 57)) ||
+			((uiinput >= 97) && (uiinput <= 123))) &&
+			(uiinput < i)) {
+			
+			if (uiinput <= 57)
+				i = uiinput - 48;
+			else
+				i = uiinput - 87;
+			
+			if (n > 0) {
+				/* Delete category code for n (number of categories defined for current feed) > 0 */
+
+				/* Decrease i by one while walking the category linked list.
+				   If we hit 0, break. category->name will now contain the category we want to remove. */
+				for (category = current_feed->feedcategories; category != NULL; category = category->next_ptr) {
+					if (i == 1)
+						break;
+					i--;
+				}
+			
+				/* Delete entry from feed categories. */
+				FeedCategoryDelete (current_feed, category->name);
+			} else {
+				/* If n == 0 add category with number i to feed. */
+				for (cur_ptr = first_category; cur_ptr != NULL; cur_ptr = cur_ptr->next_ptr) {
+					if (i == 1)
+						break;
+					i--;
+				}
+				
+				/* Add category to current feed */
+				FeedCategoryAdd (current_feed, cur_ptr->name);
+			}
+		}
+	}
+
+}
+
+/* This function lets the user assign color labels to a category. */
+void CatConf (void) {
+	UIStatus ("Please write me! -- CatConf() 2003 A.D.", 0);
+	getch();
+}
+
+/* Allocates and returns a filter string the user has chosen from the list of
+   available categories.
+   If no filter is chosen a NULL pointer is returned. */
+char * DialogGetCategoryFilter (void) {
+	int i, nglobal, y;
+	char *filterstring = NULL;
+	int uiinput;
+	struct categories *cur_ptr;
+	
+	/* Determine number of global categories. */
+	nglobal = 0;
+	for (cur_ptr = first_category; cur_ptr != NULL; cur_ptr = cur_ptr->next_ptr) {
+		nglobal++;
+	}
+	
+	UISupportDrawBox ((COLS/2)-35, 2, (COLS/2)+35, nglobal+4);
+	
+	attron (WA_REVERSE);
+	mvaddstr (3, (COLS/2)-(strlen(_("Select filter to apply"))/2), _("Select filter to apply"));
+	
+	i = 49;
+	y = 3;
+	for (cur_ptr = first_category; cur_ptr != NULL; cur_ptr = cur_ptr->next_ptr) {
+		mvprintw (y+1, (COLS/2)-33, "%c. %s", i, cur_ptr->name);
+		y++;
+		i++;
+		/* Fast forward to 'a' if we hit ASCII 58 ':' */
+		if (i == 58)
+			i += 39;
+	}
+			
+	UIStatus (_("Select a category number or any other key to reset filters."), 0);
+	
+	refresh();
+		
+	uiinput = getch();
+
+	/* If uiinput is 1-9,a-z
+	   If uiinput < i (ASCII of max selectable element) ignore event. */
+	if ((((uiinput >= 49) && (uiinput <= 57)) ||
+		((uiinput >= 97) && (uiinput <= 123))) &&
+		(uiinput < i)) {
+		
+		if (uiinput <= 57)
+			i = uiinput - 48;
+		else
+			i = uiinput - 87;
+		
+		for (cur_ptr = first_category; cur_ptr != NULL; cur_ptr = cur_ptr->next_ptr) {
+			if (i == 1)
+				break;
+			i--;
+		}
+		filterstring = strdup (cur_ptr->name);
+	}
+	
+	return filterstring;
+}
