@@ -35,6 +35,7 @@
 #include "conversions.h"
 #include "dialog.h"
 #include "ui-support.h"
+#include "categories.h"
 
 #ifdef SUN
 #	include "os-support.h"
@@ -43,6 +44,12 @@
 
 extern char *browser;
 extern struct keybindings keybindings;
+extern struct color color;
+extern int use_colors;
+
+struct feed *first_bak = NULL;	/* Backup first pointer for filter mode.
+								   Needs to be global so it can be used in the signal handler.
+								   Must be set to NULL by default and whenever it's not used anymore! */
 
 /* Scrollable feed->description. */
 struct scrolltext {
@@ -51,42 +58,6 @@ struct scrolltext {
 	struct scrolltext * prev_ptr;
 };
 
-/* Init the ncurses library. */
-void InitCurses (void) {
-	initscr();
-
-	keypad (stdscr, TRUE);	/* Activate keypad 'cuz some howto told me to do so. */
-	
-	cbreak();				/* No buffering. */
-	noecho();				/* Do not echo typed chars onto the screen. */
-	clear();
-	curs_set(0);			/* Hide cursor. */
-	refresh();
-}
-
-
-/* Print text in statusbar. */
-void UIStatus (char * text, int delay) {
-	int i;
-			
-	attron (WA_REVERSE);
-	/* Clear statusbar. */
-	for (i = 0; i < COLS-1; i++) {
-		mvaddch (LINES-1, i, ' ');
-	}
-	
-	mvaddnstr (LINES-1, 1, text, COLS-2);
-	
-	/* attroff is called here. If the calling function had it switched on,
-	   switch it on again there! */
-	attroff (WA_REVERSE);
-	refresh();
-	
-	if (delay)
-		sleep (delay);
-}
-
-
 /* View newsitem in scrollable window.
  * Speed of this code has been greatly increased in 1.2.1
  */
@@ -97,6 +68,7 @@ void UIDisplayItem (struct newsitem * current_item, struct feed * current_feed)
 	char *syscallnull = NULL;		/* gcc will issue "might be used uninitialized" otherwise. */
 	char *escapetext = NULL;
 	char *tmp;
+	char tmpstr[512];
 	char *feeddescscreenlength;		/* Used to print a max of COLS-something
 									   for feed->desc onto the screen. */
 	char *newtext;
@@ -158,13 +130,8 @@ void UIDisplayItem (struct newsitem * current_item, struct feed * current_feed)
 		
 		move (0, 0);
 		clrtobot();
-
-		attron (WA_REVERSE);
-		for (i = 0; i < COLS; i++) {
-			mvaddch (0, i, ' ');
-		}
-		mvprintw (0, 1, "* Snownews %s", VERSION);
-		attroff (WA_REVERSE);
+		
+		UISupportDrawHeader(NULL);
 
 		/* If feed doesn't have a description print title instead. */
 		attron (WA_BOLD);
@@ -287,18 +254,33 @@ void UIDisplayItem (struct newsitem * current_item, struct feed * current_feed)
 			}
 		}
 		
+		/* Apply color style. */
+		if (use_colors) {
+			attron (COLOR_PAIR(3));
+			if (color.urljumpbold)
+				attron (WA_BOLD);
+		} else
+			attron (WA_BOLD);
 
-		attron (WA_BOLD);
 		mvprintw (LINES-2, 1, "-> %s", syscall);
-		attroff (WA_BOLD);
+		
+		/* Disable color style. */
+		if (use_colors) {
+			attroff (COLOR_PAIR(3));
+			if (color.urljumpbold)
+				attroff (WA_BOLD);
+		} else
+			attroff (WA_BOLD);
 		
 		/* Set current item to read. */
 		current_item->readstatus = 1;
 		
-		UIStatus (_("Press 'q' or Enter to return to previous screen. Hit 'h' for help screen."), 0);
+		snprintf (tmpstr, sizeof(tmpstr), _("Press '%c' or Enter to return to previous screen. Hit '%c' for help screen."),
+			keybindings.quit, keybindings.help);
+		UIStatus (tmpstr, 0);
 		uiinput = getch();
 		
-		if (uiinput == 'h')
+		if (uiinput == keybindings.help)
 			UIDisplayItemHelp();
 		if ((uiinput == '\n') ||
 			(uiinput == keybindings.quit)) {
@@ -372,7 +354,7 @@ void UIDisplayItem (struct newsitem * current_item, struct feed * current_feed)
 					linenumber--;
 			}
 		}
-		if (uiinput == 'A')
+		if (uiinput == keybindings.about)
 			UIAbout();
 		if (uiinput == KEY_UP) {
 			if (linenumber >= 1)
@@ -416,7 +398,7 @@ void UIDisplayItem (struct newsitem * current_item, struct feed * current_feed)
 
 void UIDisplayFeed (struct feed * current_feed) {
 	char *tmp;
-	char *syscall, *syscallnull, *escapetext;
+	char *syscall, *syscallnull, *escapetext, *syscallitem;
 	int uiinput = 0;
 	int typeahead = 0;				/* Typeahead enabled? */
 	int skip = 0;					/* # of lines to skip (for typeahead) */
@@ -462,6 +444,8 @@ void UIDisplayFeed (struct feed * current_feed) {
 		syscall = malloc (len);
 		snprintf (syscall, len, browser, "");
 	}
+	/* We don't need escapetext anymore for now. */
+	free (escapetext);
 	
 	/* Set highlighted to first_ptr at the beginning.
 	   Otherwise bad things (tm) will happen. */
@@ -494,8 +478,8 @@ void UIDisplayFeed (struct feed * current_feed) {
 	}
 	/* If *highlighted points to a read entry now we have hit the last
 	   entry and there are no unread items anymore. Restore the original
-	   location in this case. */
-	/* Check if we have no items at all! */
+	   location in this case.
+	   Check if we have no items at all! */
 	if (highlighted != NULL) {
 		if (highlighted->readstatus == 1) {
 			highlighted = tmp_highlighted;
@@ -515,13 +499,30 @@ void UIDisplayFeed (struct feed * current_feed) {
 			clrtobot();
 		}
 		
-		/* This clears/inverts the first line and prints the app name/version. */
-		attron (WA_REVERSE);
-		for (i = 0; i < COLS; i++) {
-			mvaddch (0, i, ' ');
+		/* Construct url for direct opening */
+		/* NULL pointer check! Item may not contain an url at all. */
+		escapetext = NULL;
+		syscallitem = NULL;
+		if (highlighted->link != NULL) {
+			if ((strstr(highlighted->link, "'")) == NULL) {
+				len = strlen(highlighted->link) + 3;
+				escapetext = malloc (len);
+				snprintf (escapetext, len, "\'%s\'", highlighted->link);
+				len = strlen(browser) + len + 1;
+				syscallitem = malloc (len);
+				snprintf (syscallitem, len, browser, escapetext);
+			} else {
+				len = strlen(browser) + 1;
+				syscallitem = malloc (len);
+				snprintf (syscallitem, len, browser, "");
+			}
+		} else {
+			len = strlen(browser) + 1;
+			syscallitem = malloc (len);
+			snprintf (syscallitem, len, browser, "");
 		}
-		mvprintw (0, 1, "* Snownews %s", VERSION);
-		attroff (WA_REVERSE);
+		
+		UISupportDrawHeader(NULL);
 		
 		if (typeahead) {
 			/* This resets the offset for every typeahead loop. */
@@ -538,7 +539,9 @@ void UIDisplayFeed (struct feed * current_feed) {
 							first_scr_ptr = first_scr_ptr->next_ptr;
 				}
 				if (searchlen > 0) {
-					if (strncmp (search, cur_ptr->title, searchlen) == 0) {
+					/* if (strncmp (search, cur_ptr->title, searchlen) == 0) {*/
+					/* Substring match. */
+					if (strstr (cur_ptr->title, search) != NULL) {
 						found = 1;
 						highlighted = cur_ptr;
 						if (skip >= 1) {
@@ -563,7 +566,7 @@ void UIDisplayFeed (struct feed * current_feed) {
 				first_scr_ptr = savestart_first;
 			}
 		}
-		
+
 		attron (WA_BOLD);
 		if (current_feed->description != NULL) {
 			/* Print a max of COLS-something chars.
@@ -606,8 +609,16 @@ void UIDisplayFeed (struct feed * current_feed) {
 			move (ypos, 0);
 			clrtoeol();
 			
-			if (curitem_ptr->readstatus != 1)
-				attron (WA_BOLD);
+			if (curitem_ptr->readstatus != 1) {
+				/* Apply color style. */
+				if (use_colors) {
+					attron (COLOR_PAIR(2));
+					if (color.newitemsbold)
+						attron (WA_BOLD);
+				} else
+					attron (WA_BOLD);
+			}
+
 			if (curitem_ptr == highlighted) {
 				highlightnum = itemnum;
 				attron (WA_REVERSE);
@@ -639,24 +650,47 @@ void UIDisplayFeed (struct feed * current_feed) {
 			ypos++;
 			if (curitem_ptr == highlighted)
 				attroff (WA_REVERSE);
-			if (curitem_ptr->readstatus != 1)
-				attroff (WA_BOLD);
+			if (curitem_ptr->readstatus != 1) {
+				/* Disable color style. */
+				if (use_colors) {
+					attroff (COLOR_PAIR(2));
+					if (color.newitemsbold)
+						attroff (WA_BOLD);
+				} else
+					attroff (WA_BOLD);
+			}
 				
 			if (itemnum >= LINES-7)
 				break;
 			itemnum++;
 		}
-		/* refresh(); */ /* Not needed since UIStatus does this for us. */
 		
-		attron (WA_BOLD);
+		/* Apply color style. */
+		if (use_colors) {
+			attron (COLOR_PAIR(3));
+			if (color.urljumpbold == 1)
+				attron (WA_BOLD);
+		} else
+			attron (WA_BOLD);
+			
 		mvprintw (LINES-2, 1, "-> %s", syscall);
-		attroff (WA_BOLD);
+		
+		/* Disable color style. */
+		if (use_colors) {
+			attroff (COLOR_PAIR(3));
+			if (color.urljumpbold == 1)
+				attroff (WA_BOLD);
+		} else
+			attroff (WA_BOLD);
 		
 		if (typeahead) {
 			snprintf (tmpstr, sizeof(tmpstr), "-> %s", search);
 			UIStatus (tmpstr, 0);
-		} else
-			UIStatus (_("Press 'q' to return to main menu, 'h' to show help."), 0);
+		} else {
+			snprintf (tmpstr, sizeof(tmpstr), _("Press '%c' to return to main menu, '%c' to show help."),
+				keybindings.quit, keybindings.help);
+			UIStatus (tmpstr, 0);
+		}
 		
 		uiinput = getch();
 		
@@ -681,13 +715,16 @@ void UIDisplayFeed (struct feed * current_feed) {
 				}
 			}
 		} else {
-			if (uiinput == 'h')
+			if (uiinput == keybindings.help)
 				UIDisplayFeedHelp();
 			if (uiinput == keybindings.quit) {
 				/* Only free these when we return!
 				   Otherwise this would introduce multifrees. 8) */
 				free (syscall);
-				free (escapetext);
+				if (syscallitem != NULL)
+					free (syscallitem);
+				if (escapetext != NULL)
+					free (escapetext);
 				return;
 			}
 			if ((uiinput == KEY_UP) ||
@@ -743,6 +780,10 @@ void UIDisplayFeed (struct feed * current_feed) {
 				first_scr_ptr = highlighted;
 			}
 			if (uiinput == keybindings.reload) {
+				if (first_bak != NULL) {
+					UIStatus (_("Please deactivate the category filter before using this function."), 2);
+					continue;
+				}
 				UpdateFeed (current_feed);
 				highlighted = current_feed->items;
 				/* Reset first_scr_ptr if reloading. */
@@ -773,18 +814,40 @@ void UIDisplayFeed (struct feed * current_feed) {
 				free (tmp);
 				free (syscallnull);
 			}
+			if (uiinput == keybindings.urljump2) {
+				/* Copy "Executing lynx URL" into tmp and show it,
+				   copy "lynx URL 2>/dev/null" into syscallnull and execute via system. */
+				len = strlen(syscallitem) + strlen(_("Executing %s")) + 1;
+				tmp = malloc (len);
+				snprintf (tmp, len, _("Executing %s"), syscallitem);
+				UIStatus (tmp, 0);
+				len = len + 13;
+				syscallnull = malloc (len);
+				snprintf (syscallnull, len, "%s 2>/dev/null", syscallitem);
+				
+				/* Switch on the cursor. */
+				curs_set(1);
+				
+				system(syscallnull);
+				
+				InitCurses();
+				
+				/* Hide cursor again. */
+				curs_set(0);
+				
+				free (tmp);
+				free (syscallnull);
+			}
 			if (uiinput == keybindings.markread) {
 				/* Mark everything read. */
 				for (markasread = current_feed->items; markasread != NULL; markasread = markasread->next_ptr) {
 					markasread->readstatus = 1;
 				}
-			}		
-			if (uiinput == 'A')
+			}
+			if (uiinput == keybindings.about)
 				UIAbout();
 			if (uiinput == keybindings.feedinfo)
 				FeedInfo(current_feed);
-			
-			/* DON NOT free escapetext and syscall here. It would result in multiple frees! */
 		}
 		
 		if (uiinput == '\n') {
@@ -793,6 +856,7 @@ void UIDisplayFeed (struct feed * current_feed) {
 				free (search);
 				typeahead = 0;
 				curs_set(0);
+				skip = 0;
 			}
 			/* Check if we have no items at all!
 			   Don't even try to view a non existant item. */
@@ -843,11 +907,14 @@ void UIDisplayFeed (struct feed * current_feed) {
 				} else {
 					found = 0;
 					for (cur_ptr = current_feed->items; cur_ptr != NULL; cur_ptr = cur_ptr->next_ptr) {
-						if (strncmp (search, cur_ptr->title, searchlen) == 0) {
+						/*if (strncmp (search, cur_ptr->title, searchlen) == 0) {
 							if (strncmp (search, cur_ptr->title, searchlen) == 0) {
 								found++;
 							}
-						}
+						}*/
+						/* Substring match. */
+						if (strstr (cur_ptr->title, search) != NULL)
+							found++;
 					}
 					if (skip == found-1) {
 						skip = 0;
@@ -885,24 +952,28 @@ void UIDisplayFeed (struct feed * current_feed) {
 			search = realloc (search, 2);
 			memset (search, 0, 2);
 		}
-
 		
+		if (escapetext != NULL)
+			free (escapetext);
+		if (syscallitem != NULL)
+			free (syscallitem);
 	}	
 }
 
 void UIMainInterface (void) {
 	int i, ypos, len;
 	int newcount;					/* Number of new articles. */
-	int uiinput = 0;
+	int uiinput = 0;				/* User input. Needs to be int to read function keys with getch */
 	int typeahead = 0;
 	int skip = 0;					/* Number of typeahead matches to skip. */
 	int skipper = 0;				/* " skipped */
 	int found = 0;					/* Number of typeahead matches. */
 	char *search = NULL;			/* Typeahead search string. */
 	int searchlen = 0;				/* " length. */
-	char tmp[1024];
+	char tmp[1024];					/* Used mostly for UIStatus. Always use snprintf! */
 	char *file, *hashme;
 	struct feed *cur_ptr;
+	struct feed *new_ptr;
 	struct newsitem *curitem_ptr;
 	struct newsitem *markasread;
 	struct feed *highlighted;
@@ -910,12 +981,15 @@ void UIMainInterface (void) {
 	struct feed *removed;			/* Removed struct tmp pointer. */
 	struct feed *savestart;
 	struct feed *savestart_first = NULL;
+	struct feedcategories *category;
 	int itemnum;					/* Num of items on screen. Max is LINES-4 */
 	int highlightnum = 1;
 	int columns;
 	int count;
 	char *maxlinesize;
 	int forceredraw = 0;
+	char *filter = NULL;			/* Category filter. Must be NULL when not used! */
+	int filteractivated = 0;		/* Set if filter was activated */
 
 	first_scr_ptr = first_ptr;
 	highlighted = first_ptr;
@@ -937,15 +1011,68 @@ void UIMainInterface (void) {
 			move (0,0);
 			clrtobot();
 		}
-		
-		attron (WA_REVERSE);	
-		for (i = 0; i < COLS; i++) {
-			mvaddch (0, i, ' ');
+
+		UISupportDrawHeader(filter);
+				
+		/* If a filter is defined we need to make copy of the pointers in
+		   struct feed and work on that.
+		   Build a new list only with items matching current filter. */
+		/* This should be moved to its own function in ui-support.c! */
+		if (filteractivated) {
+			if (first_bak == NULL)
+				first_bak = first_ptr;
+			first_ptr = NULL;
+			for (cur_ptr = first_bak; cur_ptr != NULL; cur_ptr = cur_ptr->next_ptr) {
+				found = 0;
+				if (cur_ptr->feedcategories != NULL) {
+					for (category = cur_ptr->feedcategories; category != NULL; category = category->next_ptr) {
+						if (strcasecmp (category->name, filter) == 0) {
+							/* Match found */
+							found = 1;
+						}
+					}
+					if (!found)
+						continue;
+				} else
+					continue;
+				if (found) {
+					new_ptr = malloc (sizeof(struct feed));
+					/* These structs will get leaked!
+					   Yeah, yeah, it is on my todo list...
+					   Though valgrind doesn't report them as leaked. */
+
+					/* Duplicate pointers */
+					new_ptr->feedurl = cur_ptr->feedurl;
+					new_ptr->feed = cur_ptr->feed;
+					new_ptr->title = cur_ptr->title;
+					new_ptr->link = cur_ptr->link;
+					new_ptr->description = cur_ptr->description;
+					new_ptr->lastmodified = cur_ptr->lastmodified;
+					new_ptr->lasthttpstatus = cur_ptr->lasthttpstatus;
+					new_ptr->items = cur_ptr->items;
+					new_ptr->problem = cur_ptr->problem;
+					new_ptr->override = cur_ptr->override;
+					new_ptr->original = cur_ptr->original;
+					new_ptr->feedcategories = cur_ptr->feedcategories;
+
+					/* Add to bottom of pointer chain. */
+					new_ptr->next_ptr = NULL;
+					if (first_ptr == NULL) {
+						new_ptr->prev_ptr = NULL;
+						first_ptr = new_ptr;
+					} else {
+						new_ptr->prev_ptr = first_ptr;
+						while (new_ptr->prev_ptr->next_ptr != NULL)
+							new_ptr->prev_ptr = new_ptr->prev_ptr->next_ptr;
+						new_ptr->prev_ptr->next_ptr = new_ptr;
+					}
+				}
+			}
+			first_scr_ptr = first_ptr;
+			highlighted = first_scr_ptr;
+			filteractivated = 0;
 		}
-		//mvaddnstr (0, 0, " ", COLS-1);
-		mvprintw (0, 1, "* Snownews %s", VERSION);
-		attroff (WA_REVERSE);
-	
+		
 		ypos = 2;
 		itemnum = 1;
 	
@@ -964,7 +1091,10 @@ void UIMainInterface (void) {
 							first_scr_ptr = first_scr_ptr->next_ptr;
 				}
 				if (searchlen > 0) {
-					if (strncmp (search, cur_ptr->title, searchlen) == 0) {
+					/* Exact match from beginning of line. */
+					/* if (strncmp (search, cur_ptr->title, searchlen) == 0) { */
+					/* Substring match. */
+					if (strstr (cur_ptr->title, search) != NULL) {
 						found = 1;
 						highlighted = cur_ptr;
 						if (skip >= 1) {
@@ -995,6 +1125,24 @@ void UIMainInterface (void) {
 			move (ypos, 0);
 			clrtoeol();
 			
+			/* Determine number of new items in feed. */
+			newcount = 0;
+			for (curitem_ptr = cur_ptr->items; curitem_ptr != NULL; curitem_ptr = curitem_ptr->next_ptr) {
+				if (curitem_ptr->readstatus == 0)
+					newcount++;
+			}
+			
+			/* Switch on color if newcount>0 */
+			//if (newcount > 0) {
+				/* Print feed name in newitems color. */
+			//	if (use_colors) {
+			//		attron (COLOR_PAIR(2));
+			//		if (color.newitemsbold)
+			//			attron (WA_BOLD);
+			//	}
+			//}
+			
+			/* Make highlight if we are the highlighted feed */
 			if (cur_ptr == highlighted) {
 				highlightnum = itemnum;
 				attron (WA_REVERSE);
@@ -1002,7 +1150,10 @@ void UIMainInterface (void) {
 					mvaddch (ypos, i, ' ');
 			}
 			
-			columns = COLS-9-strlen(_("new"));
+			if (cur_ptr->feedcategories != NULL)
+				columns = COLS-26-strlen(_("new"));
+			else
+				columns = COLS-9-strlen(_("new"));
 			if (strlen(cur_ptr->title) > columns) {
 				maxlinesize = malloc(columns+1);
 				strncpy (maxlinesize, cur_ptr->title, columns);
@@ -1017,34 +1168,50 @@ void UIMainInterface (void) {
 					mvaddch (ypos, 0, '!');
 			}
 			
-			newcount = 0;
-			for (curitem_ptr = cur_ptr->items; curitem_ptr != NULL; curitem_ptr = curitem_ptr->next_ptr) {
-				if (curitem_ptr->readstatus == 0)
-					newcount++;
-			}
+			//newcount = 0;
+			//for (curitem_ptr = cur_ptr->items; curitem_ptr != NULL; curitem_ptr = curitem_ptr->next_ptr) {
+			//	if (curitem_ptr->readstatus == 0)
+			//		newcount++;
+			//}
+			
 			/* Execute this _after_ the for loop. Otherwise it'll suck CPU like crazy! */
 			if (newcount != 0) {
 				snprintf (tmp, sizeof(tmp), _("%3d new"), newcount);
 				mvaddstr (ypos, COLS-5-strlen(_("new")), tmp);
 			}
+			
+			if (cur_ptr->feedcategories != NULL)
+				mvaddnstr (ypos, COLS-21-strlen(_("new")), cur_ptr->feedcategories->name, 15);
+						
 			ypos++;
+			
 			if (cur_ptr == highlighted)
 				attroff (WA_REVERSE);
 			
+			/* Switch off color mode again. */
+			//if (newcount > 0) {
+			//	if (use_colors) {
+			//		attroff (COLOR_PAIR(2));
+			//		if (color.newitemsbold)
+			//			attroff (WA_BOLD);
+			//	}
+			//}
+			
 			if (itemnum >= LINES-4)
 				break;
 			itemnum++;
 		}
-		/* refresh(); */ /* Not needed since UIStatus does this for us. */
 
 		if (typeahead) {
 			snprintf (tmp, sizeof(tmp), "-> %s", search);
 			UIStatus (tmp, 0);
-		} else
-			UIStatus (_("Press 'h' for help window."), 0);
+		} else {
+			snprintf (tmp, sizeof(tmp), _("Press '%c' for help window."), keybindings.help);
+			UIStatus (tmp, 0);
+		}
 		
 		uiinput = getch();
-				
+		
 		if (typeahead) {
 			/* Only match real characters. */
 			if ((uiinput >= 32) && (uiinput <= 126)) {
@@ -1065,6 +1232,9 @@ void UIMainInterface (void) {
 			}
 		} else {
 			if (uiinput == keybindings.quit) {
+				/* Restore original first_ptr if filter is defined! */
+				if (filter != NULL)
+					first_ptr = first_bak;
 #ifdef QUIT_CONFIRM
 				snprintf (tmp, sizeof(tmp), _("Exit Snownews? Press '%c' again to quit."), keybindings.quit);
 				UIStatus (tmp, 0);
@@ -1073,20 +1243,24 @@ void UIMainInterface (void) {
 					MainQuit (NULL, NULL);
 			}
 			if (uiinput == keybindings.reload) {
+				if (filter != NULL) {
+					UIStatus (_("Please deactivate the category filter before using this function."), 2);
+					continue;
+				}
 				/* Refresh highlighted feed. */
 				if (UpdateFeed (highlighted) != 0) {
-					if (lasthttpstatus == 0) {
+					if (highlighted->lasthttpstatus == 0) {
 						UIStatus (_("Aborted."), 1);
 						continue;
 					}
-					if (lasthttpstatus == 304)
+					if (highlighted->lasthttpstatus == 304)
 						continue;
-					if (lasthttpstatus != 200) {
-						snprintf (tmp, sizeof(tmp), _("HTTP error %d - Could not update feed."), lasthttpstatus);
+					if (highlighted->lasthttpstatus != 200) {
+						snprintf (tmp, sizeof(tmp), _("HTTP error %d - Could not update feed."), highlighted->lasthttpstatus);
 						UIStatus (tmp, 2);
 						continue;
 					}
-					if (lasthttpstatus != 0) {
+					if (highlighted->lasthttpstatus != 0) {
 						snprintf (tmp, sizeof(tmp), _("Could not update feed %s - %s"), highlighted->title, strerror(errno));
 						UIStatus (tmp, 2);
 					}
@@ -1094,10 +1268,19 @@ void UIMainInterface (void) {
 				forceredraw = 1;
 			}
 			if (uiinput == keybindings.reloadall) {
+				if (filter != NULL) {
+					UIStatus (_("Please deactivate the category filter before using this function."), 2);
+					continue;
+				}
 				UpdateAllFeeds();
 				forceredraw = 1;
 			}
 			if (uiinput == keybindings.addfeed) {
+				/* If filter is selected add feed with current category. */
+				if (filter != NULL) {
+					UIStatus (_("Please deactivate the category filter before using this function."), 2);
+					continue;
+				}
 				switch (UIAddFeed()) {
 					case 0:
 						UIStatus (_("Successfully added new item..."), 1);
@@ -1123,12 +1306,21 @@ void UIMainInterface (void) {
 				}
 				forceredraw = 1;
 			}
-			if (uiinput == 'h')
+			if (uiinput == keybindings.help)
 				UIHelpScreen();
 			if (uiinput == keybindings.deletefeed) {
+				/* This should be moved to its own function in ui-support.c! */
+				if (filter != NULL) {
+					UIStatus (_("Please deactivate the category filter before using this function."), 2);
+					continue;
+				}
+				/* If the deleted feed was the last one of a specific category,
+				   remove this category from the global list. */
 				if (UIDeleteFeed(highlighted->title) == 1) {
 					/* Do it! */
 					removed = highlighted;
+					/* Save current highlight position. */
+					cur_ptr = highlighted;
 					
 					/* Remove cachefile from filesystem. */
 				
@@ -1158,15 +1350,30 @@ void UIMainInterface (void) {
 							first_ptr = NULL;
 							first_scr_ptr = NULL;
 						}
+						/* Set new highlighted to first_ptr again. */
+						cur_ptr = first_ptr;
 					} else if (highlighted->next_ptr == NULL) {
 						/* last element */
+						/* Set new highlighted to element before deleted one. */
+						cur_ptr = highlighted->prev_ptr;
+						/* If highlighted was first line move first line upward in pointer chain. */
+						if (highlighted == first_scr_ptr)
+							first_scr_ptr = first_scr_ptr->prev_ptr;
+						
 						highlighted->prev_ptr->next_ptr = NULL;
 					} else {
-						/* element inside list */
+						/* element inside list */		
+						/* Set new highlighted to element after deleted one. */
+						cur_ptr = highlighted->next_ptr;
+						/* If highlighted was last line, move first line downward in pointer chain. */
+						if (highlighted == first_scr_ptr)
+							first_scr_ptr = first_scr_ptr->next_ptr;
+						
 						highlighted->prev_ptr->next_ptr = highlighted->next_ptr;
 						highlighted->next_ptr->prev_ptr = highlighted->prev_ptr;
 					}
-					highlighted = first_ptr;
+					/* Put highlight to new highlight position. */
+					highlighted = cur_ptr;
 				
 					/* free (removed) pointer */
 					if (removed->items != NULL) {
@@ -1263,6 +1470,11 @@ void UIMainInterface (void) {
 				first_scr_ptr = highlighted;
 			}
 			if (uiinput == keybindings.moveup) {
+				/* This function is deactivated when a filter is active. */
+				if (filter != NULL) {
+					UIStatus (_("You cannot move items while a category filter is defined!"), 2);
+					continue;
+				}
 				/* Move item up. */
 				if (highlighted->prev_ptr != NULL) {
 					SwapPointers (highlighted, highlighted->prev_ptr);
@@ -1274,6 +1486,11 @@ void UIMainInterface (void) {
 				}
 			}
 			if (uiinput == keybindings.movedown) {
+				/* This function is deactivated when a filter is active. */
+				if (filter != NULL) {
+					UIStatus (_("You cannot move items while a category filter is defined!"), 2);
+					continue;
+				}
 				/* Move item down. */
 				if (highlighted->next_ptr != NULL) {
 					SwapPointers (highlighted, highlighted->next_ptr);
@@ -1290,15 +1507,31 @@ void UIMainInterface (void) {
 				}
 			}
 			if (uiinput == keybindings.markread) {
-				for (cur_ptr = first_scr_ptr; cur_ptr != NULL; cur_ptr = cur_ptr->next_ptr) {
+				/* This function is safe for using in filter mode, because it only
+				   changes int values. */
+				for (cur_ptr = first_ptr; cur_ptr != NULL; cur_ptr = cur_ptr->next_ptr) {
 					for (markasread = cur_ptr->items; markasread != NULL; markasread = markasread->next_ptr) {
-						markasread->readstatus = 1;
+						/* Only mark items matching current filter read if one is active. */
+						if (filter != NULL) {
+							if (cur_ptr->feedcategories != NULL) {
+								if (strcasecmp (filter, cur_ptr->feedcategories->name) == 0) {
+									/* Matched! */
+									markasread->readstatus = 1;
+								}
+							}
+						} else
+							markasread->readstatus = 1;
 					}
 				}
 			}
-			if (uiinput == 'A')
+			if (uiinput == keybindings.about)
 				UIAbout();
 			if (uiinput == keybindings.changefeedname) {
+				/* This needs to be worked on before it works while a filter is applied! */
+				if (filter != NULL) {
+					UIStatus (_("Please deactivate the category filter before using this function."), 2);
+					continue;
+				}
 				switch (UIChangeFeedName(highlighted)) {
 					case 0:
 						/* Changed */
@@ -1315,8 +1548,68 @@ void UIMainInterface (void) {
 						break;
 				}
 			}
-			if (uiinput == keybindings.sortfeeds)
+			if (uiinput == keybindings.sortfeeds) {
+				/* Deactivate sorting function if filter is applied. */
+				if (filter != NULL) {
+					UIStatus (_("Please deactivate the category filter before using this function."), 2);
+					continue;
+				}
 				SnowSort();
+			}
+			if (uiinput == keybindings.categorize) {
+				/* This needs to be worked on before it works while a filter is applied! */
+				if (filter != NULL) {
+					UIStatus (_("Please deactivate the category filter before using this function."), 2);
+					continue;
+				}
+				CategorizeFeed (highlighted);
+			}
+			if (uiinput == keybindings.filter) {
+				/* GUI to set a filter */
+				filter = DialogGetCategoryFilter();
+				if (filter != NULL)
+					filteractivated = 1;
+				else {
+					/* If DialogGetCategoryFilter() was used to switch off filter
+					   Restore first_ptr here. */
+					if (first_bak != NULL) {
+						/* Restore first_ptr */
+						first_ptr = first_bak;
+						first_scr_ptr = first_ptr;
+						highlighted = first_ptr;
+						first_bak = NULL;
+					}
+				}
+			}
+			if (uiinput == keybindings.filtercurrent) {
+				/* Set filter to primary filter of this feed. */
+				/* Free filter if it's not empty to avoid memory leaks! */
+				if (filter != NULL) {
+					free (filter);
+					filter = NULL;
+					/* A filter was set previously, restore original first_ptr
+					   before we activate the new one. */
+					first_ptr = first_bak;
+					first_scr_ptr = first_ptr;
+					highlighted = first_ptr;
+				}
+				if (highlighted->feedcategories != NULL) {
+					filter = strdup (highlighted->feedcategories->name);
+					filteractivated = 1;
+					}
+			}
+			if (uiinput == keybindings.nofilter) {
+				/* Remove all filters. */
+				if (filter != NULL) {
+					free (filter);
+					filter = NULL;
+					/* Restore first_ptr */
+					first_ptr = first_bak;
+					first_scr_ptr = first_ptr;
+					highlighted = first_ptr;
+					first_bak = NULL;
+				}
+			}
 		}
 		
 		if (uiinput == '\n') {
@@ -1324,6 +1617,9 @@ void UIMainInterface (void) {
 			if (typeahead) {
 				free (search);
 				typeahead = 0;
+				/* Clear skip flag, otherwise type ahead breaks, because we
+				   have a skip "offset" when using it again. */
+				skip = 0;
 			}
 			/* Select this feed, open and view entries. */
 			if (highlighted != NULL)
@@ -1347,11 +1643,15 @@ void UIMainInterface (void) {
 				} else {
 					found = 0;
 					for (cur_ptr = first_ptr; cur_ptr != NULL; cur_ptr = cur_ptr->next_ptr) {
-						if (strncmp (search, cur_ptr->title, searchlen) == 0) {
+						/* Exact match at beginning of line. */
+						/*if (strncmp (search, cur_ptr->title, searchlen) == 0) {
 							if (strncmp (search, cur_ptr->title, searchlen) == 0) {
 								found++;
 							}
-						}
+						}*/
+						/* Substring match. */
+						if (strstr (cur_ptr->title, search) != NULL)
+							found++;
 					}
 					if (skip == found-1) { /* found-1 to avoid empty tab cycle. */
 						skip = 0;
