@@ -34,10 +34,12 @@
 
 #include "config.h"
 #include "interface.h"
+#include "ui-support.h"
 #include "netio.h"
 #include "main.h"
 #include "xmlparse.h"
 #include "updatecheck.h"
+#include "categories.h"
 
 #ifdef SUN
 #	include "os-support.h"
@@ -45,12 +47,16 @@
 
 struct feed *first_ptr = NULL;
 struct keybindings keybindings;
+struct color color;
+
+extern struct feed *first_bak;		/* For use with the signal handler. */
 
 char *browser;						/* Browser command. lynx is standard. */
 int versioncheck = 1;				/* Autoupdate check. 1=yes, 0=no. */
 char *proxyname;					/* Hostname of proxyserver. */
 char *useragent;					/* Snownews User-Agent string. */
 unsigned short proxyport = 0;		/* Port on proxyserver to use. */
+int use_colors = 0;					/* Default to disabled. */
 
 
 /* Deinit ncurses and quit. */
@@ -74,16 +80,25 @@ void MainQuit (char * func, char * error) {
 		
 		exit(0);
 	} else {
-		printf (_("The program has crash^H^H^H^H^H exited abnormally.\nError %s occured in function %s.\nPlease bother maintainer.\n\n"),
-			error, func);
+		printf (_("Aborting program execution!\nAn internal error occured. Snownews has quit, no changes has been saved!\n"));
+		printf (_("This shouldn't happen, please submit a bugreport to kiza@kcore.de, tell me what you where doing and include the output below in your mail.\n"));
+		printf ("----\n");
+		/* Please don't localize! I don't want to receive Japanese error messages. Thanks. :) */
+		printf ("While executing: %s\n", func);
+		printf ("Error as reported by the system: %s\n\n", error);
 		exit(1);
 	}
 }
 
 /* Signal handler function. */
 void MainSignalHandler (int sig) {
-	if (sig == SIGINT)
+	if (sig == SIGINT) {
+		/* If there is a first_bak!=NULL a filter is set. Reset first_ptr
+		   so the correct list gets written on the disk when exisiting via SIGINT. */
+		if (first_bak != NULL)
+			first_ptr = first_bak;
 		MainQuit(NULL, NULL);
+	}
 }
 
 /* http://foo.bar/address.rdf -> http:__foo.bar_address.rdf */
@@ -97,11 +112,13 @@ void Hashify (char * url) {
 	}
 }
 
+
 /* Load config and populate caches. */
 void Config (void) {
-	char file[512];				/* File locations. */
+	char file[512];					/* File locations. */
 	char filebuf[4096];
-	char *freeme, *tmpbuf;
+	char *freeme, *tmpbuf, *tmppos, *tmpstr;
+	char *categories = NULL;		/* Holds comma seperated category string. */
 	FILE *configfile;
 	struct feed *new_ptr;
 	struct stat dirtest;
@@ -134,8 +151,11 @@ void Config (void) {
 	
 	UIStatus (_("Reading configuration settings..."), 0);
 	
-	/* Read global config. */
-	/* ~/.snownews/config -> browser. */
+
+	/*******************
+	 * Default browser *
+	 *******************/
+	 
 	snprintf (file, sizeof(file), "%s/.snownews/browser", getenv("HOME"));
 	configfile = fopen (file, "r");
 	if (configfile == NULL) {
@@ -158,7 +178,10 @@ void Config (void) {
 			browser[strlen(browser)-1] = '\0';
 	}
 
-	/* ~/.snownews/urls -> ptr->feedurl. */
+	/*************
+	 * Feed list *
+	 *************/
+	
 	snprintf (file, sizeof(file), "%s/.snownews/urls", getenv("HOME"));
 	configfile = fopen (file, "r");
 	if (configfile == NULL) {
@@ -174,8 +197,6 @@ void Config (void) {
 			if (filebuf[0] == '\n')
 				break;
 			new_ptr = malloc (sizeof(struct feed));
-						
-			//new_ptr->feedurl = malloc (strlen(filebuf));
 			
 			new_ptr->feed = NULL;
 			/* Set to NULL so xmlparse.c is happy when we try to free there. */
@@ -184,28 +205,58 @@ void Config (void) {
 			new_ptr->description = NULL;
 			new_ptr->items = NULL;
 			new_ptr->lastmodified = NULL;
+			new_ptr->lasthttpstatus = 0;
 			new_ptr->override = NULL;
 			new_ptr->original = NULL;
 			new_ptr->problem = 0;
+			new_ptr->feedcategories = NULL;
+			/* Reset to NULL on every loop run! */
+			categories = NULL;
 			
-			/* If we find an override feedname. */
-			if (strstr(filebuf, "|") != NULL) {
-				tmpbuf = strdup (filebuf);
-				freeme = tmpbuf;
-				strsep (&tmpbuf, "|");
-				new_ptr->feedurl = strdup(freeme);
-				new_ptr->override = strdup(tmpbuf);
-				free (freeme);
-				/* Munch trailing newline. */
-				if (new_ptr->override[strlen(new_ptr->override)-1] == '\n')
-					new_ptr->override[strlen(new_ptr->override)-1] = '\0';
-			} else {
-				new_ptr->feedurl = malloc (strlen(filebuf));
-				strncpy (new_ptr->feedurl, filebuf, strlen(filebuf));
-				/* Munch trailing newline. */
-				if (new_ptr->feedurl[strlen(filebuf)-1] == '\n')
-					new_ptr->feedurl[strlen(filebuf)-1] = '\0';
+			/* File format is url|custom name|comma seperated categories */
+			tmpbuf = strdup (filebuf);
+			
+			/* Munch trailing newline. */
+			if (tmpbuf[strlen(tmpbuf)-1] == '\n')
+				tmpbuf[strlen(tmpbuf)-1] = '\0';
+				
+			freeme = tmpbuf;
+			tmpstr = strsep (&tmpbuf, "|");
+			
+			/* The first | was squished with \0 so we can strdup the first part. */
+			new_ptr->feedurl = strdup(freeme);
+			
+			/* Save start position of override string */
+			tmpstr = strsep (&tmpbuf, "|");
+			if (tmpstr != NULL) {
+				if (*tmpstr != '\0') {
+					new_ptr->override = strdup(tmpstr);
+				}
 			}
+			tmpstr = strsep (&tmpbuf, "|");
+			if (tmpstr != NULL) {
+				if (*tmpstr != '\0') {
+					categories = strdup(tmpstr);
+				}
+			}
+			
+			/* We don't need freeme anymore. */
+			free (freeme);
+			
+			/* Put categories into cat struct. */
+			if (categories != NULL) {
+				freeme = categories;
+				
+				while (1) {
+					tmppos = strsep (&categories, ",");
+					if (tmppos == NULL)
+						break;
+					
+					FeedCategoryAdd (new_ptr, tmppos);
+				}
+				free (freeme);
+			} else
+				new_ptr->feedcategories = NULL;
 			
 			/* Add to bottom of pointer chain. */
 			new_ptr->next_ptr = NULL;
@@ -224,7 +275,9 @@ void Config (void) {
 		fclose (configfile);
 	}
 	
-	/* Now read global keybindings. */
+	/***********************
+	 * Global keybindings. *
+	 ***********************/
 	
 	/* Define default values for keybindings. If some are defined differently
 	   in the keybindings file they will be overwritten. If some are missing or broken/wrong
@@ -242,10 +295,17 @@ void Config (void) {
 	keybindings.reload = 'r';
 	keybindings.reloadall = 'R';
 	keybindings.urljump = 'o';
+	keybindings.urljump2 = 'O';
 	keybindings.changefeedname = 'c';
 	keybindings.sortfeeds = 's';
 	keybindings.pup = 'b';
 	keybindings.pdown = ' ';
+	keybindings.categorize = 'C';
+	keybindings.filter = 'f';
+	keybindings.filtercurrent = 'g';
+	keybindings.nofilter = 'F';
+	keybindings.help = 'h';
+	keybindings.about = 'A';
 		
 	snprintf (file, sizeof(file), "%s/.snownews/keybindings", getenv("HOME"));
 	configfile = fopen (file, "r");
@@ -262,6 +322,10 @@ void Config (void) {
 		fputs ("move item down:N\n", configfile);
 		fputs ("change feedname:c\n", configfile);
 		fputs ("sort feeds:s\n", configfile);
+		fputs ("categorize feed:C\n", configfile);
+		fputs ("apply filter:f\n", configfile);
+		fputs ("only current category:g\n", configfile);
+		fputs ("remove filter:F\n", configfile);
 		fputs ("# Feed display menu bindings\n", configfile);
 		fputs ("show feedinfo:i\n", configfile);
 		fputs ("# General keybindungs\n", configfile);
@@ -271,8 +335,11 @@ void Config (void) {
 		fputs ("reload feed:r\n", configfile);
 		fputs ("mark (all) as read:m\n", configfile);
 		fputs ("open url:o\n", configfile);
+		fputs ("open item url in overview:O\n", configfile);
 		fputs ("page up:b\n", configfile);
 		fputs ("page down: \n", configfile);
+		fputs ("help menu:h\n", configfile);
+		fputs ("about:A\n", configfile);
 		fclose (configfile);
 	} else {
 		/* Read keybindings and populate keybindings struct. */
@@ -317,12 +384,26 @@ void Config (void) {
 				keybindings.reloadall = tmpbuf[0];
 			if (strcmp (freeme, "open url") == 0)
 				keybindings.urljump = tmpbuf[0];
+			if (strcmp (freeme, "open item url in overview") == 0)
+				keybindings.urljump2 = tmpbuf[0];
 			if (strcmp (freeme, "change feedname") == 0)
 				keybindings.changefeedname = tmpbuf[0];
 			if (strcmp (freeme, "page up") == 0)
 				keybindings.pup = tmpbuf[0];
 			if (strcmp (freeme, "page down") == 0)
 				keybindings.pdown = tmpbuf[0];
+			if (strcmp (freeme, "categorize feed") == 0)
+				keybindings.categorize = tmpbuf[0];
+			if (strcmp (freeme, "apply filter") == 0)
+				keybindings.filter = tmpbuf[0];
+			if (strcmp (freeme, "only current category") == 0)
+				keybindings.filtercurrent = tmpbuf[0];
+			if (strcmp (freeme, "remove filter") == 0)
+				keybindings.nofilter = tmpbuf[0];
+			if (strcmp (freeme, "help menu") == 0)
+				keybindings.help = tmpbuf[0];
+			if (strcmp (freeme, "about") == 0)
+				keybindings.about = tmpbuf[0];
 			
 			free (freeme);
 		}
@@ -334,6 +415,91 @@ void Config (void) {
 			keybindings.dfltbrowser = 'B';
 		}
 	}
+	
+	/***********************
+	 * COLOR suppport code *
+	 ***********************/
+	
+	color.changed = 0;
+	color.newitems = 5;
+	color.newitemsbold = 0;
+	color.urljump = 4;
+	color.urljumpbold = 0;
+	
+	snprintf (file, sizeof(file), "%s/.snownews/colors", getenv("HOME"));
+	configfile = fopen (file, "r");
+	if (configfile == NULL) {
+		/* Set color configfile update flag. */
+		color.changed = 1;
+	} else {
+		while (!feof(configfile)) {
+			if ((fgets (filebuf, sizeof(filebuf), configfile)) == NULL)
+				break;
+			if (strstr(filebuf, "#") != NULL)
+				continue;
+			tmpbuf = strdup(filebuf);	/* strsep only works on *ptr */
+			freeme = tmpbuf;			/* Save start pos. This is also the string we need to compare. strsep will \0 the delimiter. */
+			strsep (&tmpbuf, ":");
+
+			/* Munch trailing newline. */
+			if (tmpbuf != NULL)
+				tmpbuf[strlen(tmpbuf)-1] = 0;
+				
+			if (strcmp (freeme, "enabled") == 0) 
+				use_colors = atoi(tmpbuf);
+			if (strcmp (freeme, "new item") == 0) {
+				color.newitems = atoi(tmpbuf);
+				if (color.newitems > 7) {
+					color.newitemsbold = 1;
+				} else
+					color.newitemsbold = 0;
+			}
+			if (strcmp (freeme, "goto url") == 0) {
+				color.urljump = atoi(tmpbuf);
+				if (color.urljump > 7) {
+					color.urljumpbold = 1;
+				} else
+					color.urljumpbold = 0;
+			}
+			
+			free (freeme);
+		}
+	}
+	
+	if (use_colors) {
+		start_color();
+		
+		/* The following call will automagically implement -1 as the terminal's
+		   default color for fg/bg in init_pair. */
+		use_default_colors();
+		
+		/* Internally used color pairs.
+		   Use with WA_BOLD to get bright color (orange->yellow, gray->white, etc). */
+		init_pair (10, 1, -1);		/* red */
+		init_pair (11, 2, -1);		/* green */
+		init_pair (12, 3, -1);		/* orange */
+		init_pair (13, 4, -1);		/* blue */
+		init_pair (14, 5, -1);		/* magenta */
+		init_pair (15, 6, -1);		/* cyan */
+		init_pair (16, 7, -1);		/* gray */
+		
+		/* Default terminal color color pair */
+		init_pair (63, -1, -1);
+		
+		/* Initialize all color pairs we're gonna use. */
+		/* New item. */
+		if (color.newitemsbold == 1)
+			init_pair (2, color.newitems-8, -1);
+		else
+			init_pair (2, color.newitems, -1);
+			
+		/* Goto url line */
+		if (color.urljumpbold == 1)
+			init_pair (3, color.urljump-8, -1);
+		else
+			init_pair (3, color.urljump, -1);
+	
+	}
 }
 
 
@@ -475,6 +641,7 @@ void WriteCache (void) {
 	struct feed *cur_ptr;
 	struct newsitem *item;
 	struct stat filetest;
+	struct feedcategories *category;
 	char *encoded;
 	
 	UIStatus (_("Saving settings..."), 0);
@@ -504,10 +671,19 @@ void WriteCache (void) {
 	
 	for (cur_ptr = first_ptr; cur_ptr != NULL; cur_ptr = cur_ptr->next_ptr) {
 		fputs (cur_ptr->feedurl, configfile);
+		fputc ('|', configfile);
 		if (cur_ptr->override != NULL) {
-			fputc ('|', configfile);
 			fputs (cur_ptr->title, configfile);
 		}
+		fputc ('|', configfile);
+		if (cur_ptr->feedcategories != NULL) {
+			for (category = cur_ptr->feedcategories; category != NULL; category = category->next_ptr) {
+				fputs (category->name, configfile);
+				/* Only add a colon of we run the loop again! */
+				if (category->next_ptr != NULL)
+					fputc (',', configfile);
+			}
+		}
 		fputc ('\n', configfile);		/* Add newline character. */
 		
 		/* 
@@ -593,6 +769,33 @@ void WriteCache (void) {
 	}
 	fclose (configfile);
 	
+	/* Write color configfile if needed, */
+	if (color.changed) {
+		snprintf (file, sizeof(file), "%s/.snownews/colors", getenv("HOME"));
+		configfile = fopen (file, "w+");
+		
+		fputs ("# Snownews color definitons\n", configfile);
+		fputs ("# black:0\n", configfile);
+		fputs ("# red:1\n", configfile);
+		fputs ("# green:2\n", configfile);
+		fputs ("# orange:3\n", configfile);
+		fputs ("# blue:4\n", configfile);
+		fputs ("# magenta(tm):5\n", configfile);
+		fputs ("# cyan:6\n", configfile);
+		fputs ("# gray:7\n", configfile);
+		fputs ("# brightred:9\n", configfile);
+		fputs ("# brightgreen:10\n", configfile);
+		fputs ("# yellow:11\n", configfile);
+		fputs ("# brightblue:12\n", configfile);
+		fputs ("# brightmagenta:13\n", configfile);
+		fputs ("# brightcyan:14\n", configfile);
+		fputs ("# white:15\n", configfile);
+		fprintf (configfile, "enabled:%d\n", use_colors);
+		fprintf (configfile, "new item:%d\n", color.newitems);
+		fprintf (configfile, "goto url:%d\n", color.urljump);
+				
+		fclose (configfile);
+	}
 	
 	return;
 }
