@@ -43,6 +43,7 @@
 #include "main.h"
 #include "interface.h"
 #include "conversions.h"
+#include "ui-support.h"
 
 #ifdef SUN
 #	include "os-support.h"
@@ -50,8 +51,6 @@
 
 #define MAX_HTTP_REDIRECTS 10	/* Maximum number of redirects we will follow. */
 
-int my_socket;
-int lasthttpstatus;					/* Last HTTP server status code. */
 extern char *proxyname;				/* Hostname of proxyserver. */
 extern unsigned short proxyport;	/* Port on proxyserver to use. */
 extern char *useragent;
@@ -64,7 +63,7 @@ int connectresult;
  *					3	couldn't connect
  *                  -1  aborted by user
  */
-int NetConnect (char * host, int httpproto) {
+int NetConnect (int * my_socket, char * host, int httpproto) {
 	int retval;
 	struct sockaddr_in address;	
 	struct hostent *remotehost;
@@ -88,19 +87,19 @@ int NetConnect (char * host, int httpproto) {
 	free (uistring);
 	
 	/* Create a inet stream TCP socket. */
-	my_socket = socket (AF_INET, SOCK_STREAM, 0);
-	if (my_socket == -1) {
+	*my_socket = socket (AF_INET, SOCK_STREAM, 0);
+	if (*my_socket == -1) {
 		return 1;
 	}
 	
 	/* Set socket to nonblock mode to make it possible to interrupt the connect. */
-	fcntl(my_socket, F_SETFL, fcntl(my_socket, F_GETFL, 0) | O_NONBLOCK);
+	fcntl(*my_socket, F_SETFL, fcntl(*my_socket, F_GETFL, 0) | O_NONBLOCK);
 	
 	/* stdin is read, socket is write, so we need read/write sets. */
 	FD_ZERO(&rfdsr);
 	FD_ZERO(&rfdsw);
 	FD_SET(0, &rfdsr);
-	FD_SET(my_socket, &rfdsw);
+	FD_SET(*my_socket, &rfdsw);
 	
 	/* If proxyport is 0 we didn't execute the if http_proxy statement in main
 	   so there is no proxy. On any other value of proxyport do proxyrequests instead. */
@@ -108,7 +107,7 @@ int NetConnect (char * host, int httpproto) {
 		/* Lookup remote IP. */
 		remotehost = gethostbyname (realhost);
 		if (remotehost == NULL) {
-			close (my_socket);
+			close (*my_socket);
 			free (realhost);
 			return 2;
 		}
@@ -119,39 +118,39 @@ int NetConnect (char * host, int httpproto) {
 		memcpy (&address.sin_addr.s_addr, remotehost->h_addr_list[0], remotehost->h_length);
 			
 		/* Connect socket. */
-		connectresult = connect (my_socket, (struct sockaddr *) &address, sizeof(address));
+		connectresult = connect (*my_socket, (struct sockaddr *) &address, sizeof(address));
 		
 		/* Check if we're already connected.
 		   BSDs will return 0 on connect even in nonblock if connect was fast enough. */
 		if (connectresult != 0) {
 			/* If errno is not EINPROGRESS, the connect went wrong. */
 			if (errno != EINPROGRESS) {
-				close (my_socket);
+				close (*my_socket);
 				free (realhost);
 				return 3;
 			}
 		
 			while (1) {
-				retval = select (my_socket+1, &rfdsr, &rfdsw, NULL, NULL);
+				retval = select (*my_socket+1, &rfdsr, &rfdsw, NULL, NULL);
 				
 				if (FD_ISSET (0, &rfdsr)) {
 					if (getch() == 'z') {
-						close (my_socket);
+						close (*my_socket);
 						free (realhost);
 						return -1;
 					}
 				}
 		
-				if (FD_ISSET (my_socket, &rfdsw))
+				if (FD_ISSET (*my_socket, &rfdsw))
 					break;
 			}
 			
 			/* We get errno of connect back via getsockopt SO_ERROR (into connectresult). */
 			len = sizeof(connectresult);
-			getsockopt(my_socket, SOL_SOCKET, SO_ERROR, &connectresult, &len);
+			getsockopt(*my_socket, SOL_SOCKET, SO_ERROR, &connectresult, &len);
 			
 			if (connectresult != 0) {
-				close (my_socket);
+				close (*my_socket);
 				free (realhost);
 				return 3;
 			}
@@ -160,7 +159,7 @@ int NetConnect (char * host, int httpproto) {
 		/* Lookup proxyserver IP. */
 		remotehost = gethostbyname (proxyname);
 		if (remotehost == NULL) {
-			close (my_socket);
+			close (*my_socket);
 			free (realhost);
 			return 2;
 		}
@@ -171,37 +170,37 @@ int NetConnect (char * host, int httpproto) {
 		memcpy (&address.sin_addr.s_addr, remotehost->h_addr_list[0], remotehost->h_length);
 		
 		/* Connect socket. */
-		connectresult = connect (my_socket, (struct sockaddr *) &address, sizeof(address));
+		connectresult = connect (*my_socket, (struct sockaddr *) &address, sizeof(address));
 		
 		/* Check if we're already connected.
 		   BSDs will return 0 on connect even in nonblock if connect was fast enough. */
 		if (connectresult != 0) {
 			if (errno != EINPROGRESS) {
-				close (my_socket);
+				close (*my_socket);
 				free (realhost);
 				return 3;
 			}
 		
 			while (1) {
-				retval = select (my_socket+1, &rfdsr, &rfdsw, NULL, NULL);
+				retval = select (*my_socket+1, &rfdsr, &rfdsw, NULL, NULL);
 			
 				if (FD_ISSET (0, &rfdsr)) {
 					if (getch() == 'z') {
-						close (my_socket);
+						close (*my_socket);
 						free (realhost);
 						return -1;
 					}
 				}
 			
-				if (FD_ISSET (my_socket, &rfdsw))
+				if (FD_ISSET (*my_socket, &rfdsw))
 					break;
 			}
 			
 			len = sizeof(connectresult);
-			getsockopt(my_socket, SOL_SOCKET, SO_ERROR, &connectresult, &len);
+			getsockopt(*my_socket, SOL_SOCKET, SO_ERROR, &connectresult, &len);
 			
 			if (connectresult != 0) {
-				close (my_socket);
+				close (*my_socket);
 				free (realhost);
 				return 3;
 			}
@@ -209,7 +208,7 @@ int NetConnect (char * host, int httpproto) {
 	}
 	
 	/* Set socket back to blocking mode. */
-	fcntl(my_socket, F_SETFL, fcntl(my_socket, F_GETFL, 0) & ~O_NONBLOCK);
+	fcntl(*my_socket, F_SETFL, fcntl(*my_socket, F_GETFL, 0) & ~O_NONBLOCK);
 	
 	free (realhost);
 	return 0;
@@ -219,7 +218,7 @@ int NetConnect (char * host, int httpproto) {
 /*
  * Kiza's crufty HTTP client hack.
  */
-char * NetIO (char * host, char * url, struct feed * cur_ptr, int httpproto) {
+char * NetIO (int * my_socket, char * host, char * url, struct feed * cur_ptr, int httpproto) {
 	char netbuf[4096];			/* Network read buffer. */
 	char *body;					/* XML body. */
 	int length;
@@ -233,9 +232,9 @@ char * NetIO (char * host, char * url, struct feed * cur_ptr, int httpproto) {
 	char *tmphost;				/* Pointers needed to strsep operation. */
 	char *newhost;				/* New hostname if we need to redirect. */
 	char *newurl;				/* New document name ". */
+	char *newlocation;
 	char *tmpstring;			/* Temp pointers. */
 	char *freeme;
-	char *freemetoo;
 	char *redirecttarget;
 	fd_set rfds;
 	int retval;
@@ -245,8 +244,7 @@ char * NetIO (char * host, char * url, struct feed * cur_ptr, int httpproto) {
 	int contentlength = 0;		/* Content-Length of server reply. */
 	int len;
 	char * inflatedbody;
-
-	lasthttpstatus = 0;
+	int quirksmode = 0;
 	
 	snprintf (tmp, sizeof(tmp), _("Downloading http://%s%s..."), host, url);
 	UIStatus (tmp, 0);
@@ -257,7 +255,7 @@ char * NetIO (char * host, char * url, struct feed * cur_ptr, int httpproto) {
 	tryagain:
 	
 	/* Open socket. */	
-	stream = fdopen (my_socket, "r+");
+	stream = fdopen (*my_socket, "r+");
 	if (stream == NULL) {
 		/* This is a serious non-continueable OS error as it will probably not
 		   go away if we retry.
@@ -289,10 +287,10 @@ char * NetIO (char * host, char * url, struct feed * cur_ptr, int httpproto) {
 	/* Use select to make the connection interuptable if it should hang. */
 	FD_ZERO(&rfds);
 	FD_SET(0, &rfds);
-	FD_SET(my_socket, &rfds);
+	FD_SET(*my_socket, &rfds);
 	
 	while (1) {
-		retval = select (my_socket+1, &rfds, NULL, NULL, NULL);
+		retval = select (*my_socket+1, &rfds, NULL, NULL, NULL);
 		
 		if (FD_ISSET (0, &rfds)) {
 			if (getch() == 'z') {
@@ -301,7 +299,7 @@ char * NetIO (char * host, char * url, struct feed * cur_ptr, int httpproto) {
 			}
 		}
 		
-		if (FD_ISSET (my_socket, &rfds))
+		if (FD_ISSET (*my_socket, &rfds))
 			break;
 	}
 
@@ -322,7 +320,7 @@ char * NetIO (char * host, char * url, struct feed * cur_ptr, int httpproto) {
 	strncpy (httpstatus, tmpstatus, 3);
 	free (savestart);
 	
-	lasthttpstatus = atoi (httpstatus);
+	cur_ptr->lasthttpstatus = atoi (httpstatus);
 	
 	/* If the redirectloop was run newhost and newurl were allocated.
 	   We need to free them here. */
@@ -331,7 +329,7 @@ char * NetIO (char * host, char * url, struct feed * cur_ptr, int httpproto) {
 		free (url);
 	}
 
-	tmphttpstatus = lasthttpstatus;
+	tmphttpstatus = cur_ptr->lasthttpstatus;
 	handled = 1;
 	/* Check HTTP server response and handle redirects. */
 	do {
@@ -370,77 +368,70 @@ char * NetIO (char * host, char * url, struct feed * cur_ptr, int httpproto) {
 					   
 					   Do not touch any of the following code! :P */
 					if (strstr (netbuf, "Location") != NULL) {
-						tmpstring = malloc(strlen(netbuf)+1);
+						//tmpstring = malloc(strlen(netbuf)+1);
 						
 						redirecttarget = strdup (netbuf);
 						freeme = redirecttarget;
+						
+						/* Remove trailing \r\n from line. */
+						redirecttarget[strlen(redirecttarget)-2] = 0;
+						
 						/* In theory pointer should now be after the space char
 						   after the word "Location:" */
 						strsep (&redirecttarget, " ");
 						
+						if (strncmp(redirecttarget, "http", 4) != 0)
+							quirksmode = 1;
+						
+						/* If the Location header is invalid we need to construct
+						   a correct one here before proceeding with the program.
+						   This makes headers like
+						   "Location: fuck-the-protocol.rdf" work.
+						   In violalation of RFC1945, RFC2616. */
+						if (quirksmode) {
+							len = 7 + strlen(host) + strlen(redirecttarget) + 3;
+							newlocation = malloc(len);
+							memset (newlocation, 0, len);
+							strcat (newlocation, "http://");
+							strcat (newlocation, host);
+							if (redirecttarget[0] != '/')
+								strcat (newlocation, "/");
+							strcat (newlocation, redirecttarget);
+						} else
+							newlocation = strdup (redirecttarget);
+						
+						/* This also frees redirecttarget. */
+						free (freeme);
+						
 						/* Change cur_ptr->feedurl on 301. */
-						if (lasthttpstatus == 301) {
+						if (cur_ptr->lasthttpstatus == 301) {
 							UIStatus (_("URL points to permanent redirect, updating with new location..."), 2);
 							free (cur_ptr->feedurl);
-							/* netbuf contains \r\n */
-							/* Should review this stuff! */
-							cur_ptr->feedurl = malloc (strlen(redirecttarget)-1);
-							strncpy (cur_ptr->feedurl, redirecttarget, strlen(redirecttarget)-2);
-							cur_ptr->feedurl[strlen(redirecttarget)-2] = '\0';
+							cur_ptr->feedurl = strdup (newlocation);
 						}
-						free (freeme);
+						
+						freeme = newlocation;
+						strsep (&newlocation, "/");
+						strsep (&newlocation, "/");
+						tmphost = newlocation;
+						/* The following line \0-terminates tmphost in overwriting the first
+						   / after the hostname. */
+						strsep (&newlocation, "/");
+					
+						newhost = strdup (tmphost);
+						newlocation--;
+						newlocation[0] = '/';
+						newurl = strdup (newlocation);
 					
-						freeme = tmpstring;
-						strcpy (tmpstring, netbuf);
-						strsep (&tmpstring, "/");
-						strsep (&tmpstring, "/");
-						tmphost = tmpstring;
-						strsep (&tmpstring, "/");
-						
-						/* Don't crash here when parsing Location header of IIS or
-						   other broken servers that don't send an absoluteURI in
-						   the header. This behaviour violates RFC1945, RFC2616. */
-						if (tmphost == NULL) {
-							/* If the server didn't sent a new hostname, just copy
-							   the original host again. MAX_HTTP_REDIRECTS will
-							   ensure we don't loop endlessly. */
-							redirecttarget = strdup (netbuf);
-							freemetoo = redirecttarget;
-							strsep (&redirecttarget, " ");
-							
-							/* Recycle old hostname. */
-							newhost = strdup (host);
-							
-							/* Add / if there is none. FUCK IIS! */
-							if (redirecttarget[0] != '/') {
-								len = strlen(redirecttarget)+2;
-								newurl = malloc (len);
-								memset (newurl, 0, len);
-								newurl[0] = '/';
-								strncat (newurl, redirecttarget, len-1);
-							} else
-								newurl = strdup (redirecttarget);
-							
-							/* Munch trailing \r\n */
-							newurl[strlen(newurl)-2] = '\0';
-							
-							free (freemetoo);
-						} else {
-							newhost = strdup (tmphost);
-						
-							tmpstring--;
-							tmpstring[0] = '/';
-							newurl = strdup (tmpstring);
-							newurl[strlen(newurl)-2] = '\0';
-						}
 						free (freeme);
 					
 						/* Close connection. */	
 						fclose (stream);
 					
 						/* Reconnect to server. */
-						if ((NetConnect (newhost, httpproto)) != 0) {
-							MainQuit (_("Reconnecting for redirect"), strerror(errno));
+						if ((NetConnect (my_socket, newhost, httpproto)) != 0) {
+							/* Add error handling/reporting. */
+							return NULL;
 						}
 					
 						host = newhost;
@@ -483,10 +474,10 @@ char * NetIO (char * host, char * url, struct feed * cur_ptr, int httpproto) {
 		/* Use select to make the connection interuptable if it should hang. */
 		FD_ZERO(&rfds);
 		FD_SET(0, &rfds);
-		FD_SET(my_socket, &rfds);
+		FD_SET(*my_socket, &rfds);
 		
 		while (1) {
-			retval = select (my_socket+1, &rfds, NULL, NULL, NULL);
+			retval = select (*my_socket+1, &rfds, NULL, NULL, NULL);
 			
 			if (FD_ISSET (0, &rfds)) {
 				if (getch() == 'z') {
@@ -495,10 +486,10 @@ char * NetIO (char * host, char * url, struct feed * cur_ptr, int httpproto) {
 				}
 			}
 			
-			if (FD_ISSET (my_socket, &rfds))
+			if (FD_ISSET (*my_socket, &rfds))
 				break;
 		}
-	
+
 		/* Max line length of sizeof(netbuf) is assumed here.
 		   If header has longer lines than 4096 bytes something may go wrong. :) */
 		if ((fgets (netbuf, sizeof(netbuf), stream)) == NULL)
@@ -516,7 +507,7 @@ char * NetIO (char * host, char * url, struct feed * cur_ptr, int httpproto) {
 		}
 		/* Get last modified date. This is only relevant on HTTP 200. */
 		if ((strstr (netbuf, "Last-Modified") != NULL) &&
-			(lasthttpstatus == 200)) {
+			(cur_ptr->lasthttpstatus == 200)) {
 			tmpstring = strdup(netbuf);
 			freeme = tmpstring;
 			strsep (&tmpstring, ":");
@@ -572,10 +563,10 @@ char * NetIO (char * host, char * url, struct feed * cur_ptr, int httpproto) {
 		/* Use select to make the connection interuptable if it should hang. */
 		FD_ZERO(&rfds);
 		FD_SET(0, &rfds);
-		FD_SET(my_socket, &rfds);
+		FD_SET(*my_socket, &rfds);
 		
 		while (1) {
-			retval = select (my_socket+1, &rfds, NULL, NULL, NULL);
+			retval = select (*my_socket+1, &rfds, NULL, NULL, NULL);
 			
 			if (FD_ISSET (0, &rfds)) {
 				if (getch() == 'z') {
@@ -584,7 +575,7 @@ char * NetIO (char * host, char * url, struct feed * cur_ptr, int httpproto) {
 				}
 			}
 			
-			if (FD_ISSET (my_socket, &rfds))
+			if (FD_ISSET (*my_socket, &rfds))
 				break;
 		}
 		
@@ -638,6 +629,7 @@ char * NetIO (char * host, char * url, struct feed * cur_ptr, int httpproto) {
 }
 
 char * DownloadFeed (char * url, struct feed * cur_ptr) {
+	int my_socket = 0;
 	int result;
 	char *host;					/* Needs to freed. */
 	char *tmphost;
@@ -687,7 +679,7 @@ char * DownloadFeed (char * url, struct feed * cur_ptr) {
 		url[strlen(url)-1] = '\0';
 	}
 	
-	result = NetConnect (host, httpproto);
+	result = NetConnect (&my_socket, host, httpproto);
 	
 	switch (result) {
 		case 1:
@@ -715,7 +707,7 @@ char * DownloadFeed (char * url, struct feed * cur_ptr) {
 			break;
 	}
 	
-	returndata = NetIO (host, url, cur_ptr, httpproto);
+	returndata = NetIO (&my_socket, host, url, cur_ptr, httpproto);
 	
 	/* url will be freed in the calling function. */
 	free (freeme);		/* This is *host. */
