@@ -257,7 +257,7 @@ int DeXML (struct feed * cur_ptr) {
 	
 	/* xmlRecoverMemory:
 	   parse an XML in-memory document and build a tree.
-       In the case the document is not Well Formed, a tree is built anyway. */
+       In case the document is not Well Formed, a tree is built anyway. */
 	doc = xmlRecoverMemory(cur_ptr->feed, strlen(cur_ptr->feed));
 	
 	if (doc == NULL)
@@ -268,6 +268,11 @@ int DeXML (struct feed * cur_ptr) {
 	   herausfindet, wie man das genau auslesen kann (jau). */
 	cur = xmlDocGetRootElement(doc);
 	
+	if (cur == NULL) {
+		xmlFreeDoc (doc);
+		return 2;
+	}
+	
 	/* berprfen, ob das Element auch wirklich <RDF> heit */
 	if (xmlStrcmp(cur->name, "RDF") == 0) {
 	
